#Import tools
import mne
import numpy as np
import cPickle as pk
from os import mkdir
from ptsa.data.readers import BaseEventReader
from ptsa.data.readers.TalReader import TalReader
from ptsa.data.readers import EEGReader
from ptsa.data.filters import MonopolarToBipolarMapper
from ptsa.data.filters import ButterworthFilter

from glob import glob
ev_files = glob('/data/events/pyFR/*_events.mat')
pyFRsubs = [d.split('/')[-1].split('_ev')[0] for d in ev_files]

MTL_labels = ['Left CA1', 'Left CA2', 'Left CA3', 'Left DG', 'Left Sub', 'Left PRC', 'Left EC', 'Left PHC',
 'Right CA1', 'Right CA2', 'Right CA3', 'Right DG', 'Right Sub', 'Right PRC', 'Right EC', 'Right PHC']

#Helper Functions
def create_baseline_events(events, start_time, end_time):
    '''
    Match recall events to matching baseline periods of failure to recall.
    Baseline events all begin at least 1000 ms after a vocalization, and end at least 1000 ms before a vocalization.
    Each recall event is matched, wherever possible, to a valid baseline period from a different list within 3 seconds
     relative to the onset of the recall period.
    Parameters:
    -----------
    events: The event structure in which to incorporate these baseline periods
    start_time: The amount of time to skip at the beginning of the session (ms)
    end_time: The amount of time within the recall period to consider (ms)
    '''

    all_events = []
    for session in np.unique(events.session):
        sess_events = events[(events.session == session)]
        irts = np.append([0], np.diff(sess_events.mstime))
        rec_events = sess_events[(sess_events.type == 'REC_WORD') & (sess_events.intrusion == 0) & (irts > 1000)]
        voc_events = sess_events[((sess_events.type == 'REC_WORD') | (sess_events.type == 'REC_WORD_VV'))]
        starts = sess_events[(sess_events.type == 'REC_START')]
        ends = sess_events[(sess_events.type == 'REC_END')]
        rec_lists = tuple(np.unique(starts.list))
        times = [voc_events[(voc_events.list == lst)].mstime for lst in rec_lists]
        start_times = starts.mstime
        end_times = ends.mstime
        epochs = free_epochs(times, 500, 2000, 1000, start=start_times, end=end_times)
        rel_times = [(t - i)[(t - i > start_time) & (t - i < end_time)] for (t, i) in
                     zip([rec_events[rec_events.list == lst].mstime for lst in rec_lists ], start_times)
                     ]
        rel_epochs = epochs - start_times[:, None]
        full_match_accum = np.zeros(epochs.shape, dtype=np.bool)
        for (i, rec_times_list) in enumerate(rel_times):
            is_match = np.empty(epochs.shape, dtype=np.bool)
            is_match[...] = False
            for t in rec_times_list:
                is_match_tmp = np.abs((rel_epochs - t)) < 3000
                is_match_tmp[i, ...] = False
                good_locs = np.where(is_match_tmp & (~full_match_accum))
                if len(good_locs[0]):
                    choice_position = np.argmin(np.mod(good_locs[0] - i, len(good_locs[0])))
                    choice_inds = (good_locs[0][choice_position], good_locs[1][choice_position])
                    full_match_accum[choice_inds] = True
        matching_epochs = epochs[full_match_accum]
        new_events = np.zeros(len(matching_epochs), dtype=sess_events.dtype).view(np.recarray)
        for i, _ in enumerate(new_events):
            new_events[i].mstime = matching_epochs[i]
            new_events[i].type = 'REC_BASE'
        new_events.recalled = 0
        merged_events = np.concatenate((sess_events, new_events)).view(np.recarray)
        merged_events.sort(order='mstime')
        for (i, event) in enumerate(merged_events):
            if event.type == 'REC_BASE':
                merged_events[i].session = merged_events[i - 1].session
                merged_events[i].list = merged_events[i - 1].list
                merged_events[i].eegfile = merged_events[i - 1].eegfile
                merged_events[i].eegoffset = merged_events[i - 1].eegoffset + (
                merged_events[i].mstime - merged_events[i - 1].mstime)
        all_events.append(merged_events)
    return np.concatenate(all_events).view(np.recarray)

def free_epochs(times, duration, pre, post, start=None, end=None):
    # (list(vector(int))*int*int*int) -> list(vector(int))
    """
    Given a list of event times, find epochs between them when nothing is happening
    Parameters:
    -----------
    times:
        An iterable of 1-d numpy arrays, each of which indicates event times
    duration: int
        The length of the desired empty epochs
    pre: int
        the time before each event to exclude
    post: int
        The time after each event to exclude
    """
    n_trials = len(times)
    epoch_times = []
    for i in range(n_trials):
        ext_times = times[i]
        if start is not None:
            ext_times = np.append([start[i]], ext_times)
        if end is not None:
            ext_times = np.append(ext_times, [end[i]])
        pre_times = ext_times - pre
        post_times = ext_times + post
        interval_durations = pre_times[1:] - post_times[:-1]
        free_intervals = np.where(interval_durations > duration)[0]
        trial_epoch_times = []
        for interval in free_intervals:
            begin = post_times[interval]
            finish = pre_times[interval + 1] - duration
            interval_epoch_times = range(int(begin), int(finish), int(duration))
            trial_epoch_times.extend(interval_epoch_times)
        epoch_times.append(np.array(trial_epoch_times))
    epoch_array = np.empty((n_trials, max([len(x) for x in epoch_times])))
    epoch_array[...] = -np.inf
    for i, epoch in enumerate(epoch_times):
        epoch_array[i, :len(epoch)] = epoch
    return epoch_array

def avg_smooth(a, winsize=2):
    
    def movingaverage(interval, window_size):
        window = np.ones(int(window_size))/float(window_size)
        return np.convolve(interval, window, 'same')
    
    from scipy.interpolate import InterpolatedUnivariateSpline, Rbf
    padnum = winsize
    
    a_ = np.concatenate([np.ones(padnum)*a[0], a, np.ones(padnum)*a[-1]]) #pad for edge effects
    a_ = movingaverage(a_, winsize) #Taking the moving average in small windows
    
    x_sm = np.arange(-1.0*padnum, len(a)+padnum)
    x_smooth = np.linspace(x_sm.min()+padnum, x_sm.max()-padnum, 152)
    a_smooth = Rbf(x_sm, a_, epsilon=0.3, function='multiquadric')
    a_smooth = a_smooth(x_smooth)
    
    return a_smooth, x_smooth


def shuffle_trials_ttest_1samp(rem, nrem, nRandomizations=500):

    from scipy.stats import ttest_1samp
    from scipy.stats import ttest_ind
    from numpy.random import permutation

    #Linearly re-map to -1, 1 scale and Fisher z-transform
    rem = np.arctanh(rem)
    nrem = np.arctanh(nrem)        

    all_dat = np.concatenate((rem, nrem), axis=0)
    group = np.hstack((np.ones(rem.shape[0])*0, np.ones(nrem.shape[0])*1))

    #Get t-stats for true data
    td = []
    td.append(np.abs(ttest_1samp(rem, 0.0, axis=0, nan_policy='omit')[0])-np.abs(ttest_1samp(nrem, 0.0, axis=0, nan_policy='omit')[0]))
    #td.append(ttest_ind(rem, nrem, axis=0, equal_var=False)[0])

    #Compute stats for each randomization
    for k in range(0, nRandomizations):
        x = permutation(group)
        rand_rem = all_dat[x==0, :, :]
        rand_nrem = all_dat[x==1, :, :]
        td.append(np.abs(ttest_1samp(rand_rem, 0.0, axis=0, nan_policy='omit')[0])-np.abs(ttest_1samp(rand_nrem, 0.0, axis=0, nan_policy='omit')[0]))
        #td.append(ttest_ind(rand_rem, rand_nrem, axis=0, equal_var=False)[0])
        print k

    td = np.array(td)

    return td  

def shuffle_trials_ttest_ind(rem, nrem, nRandomizations=500):
        
    from scipy.stats import ttest_ind
    from numpy.random import permutation

    #Linearly re-map to -1, 1 scale and Fisher z-transform
    rem = np.arctanh((rem*2.0)-1.0)
    nrem = np.arctanh((nrem*2.0)-1.0)        

    all_dat = np.concatenate((rem, nrem), axis=0)
    group = np.hstack((np.ones(rem.shape[0])*0, np.ones(nrem.shape[0])*1))

    #Get t-stats for true data
    tt = []
    tt.append(ttest_ind(rem, nrem, axis=0, equal_var=False)[0])

    #Compute stats for each randomization
    for k in range(0, nRandomizations):
        x = permutation(group)
        rand_rem = all_dat[x==0, :, :]
        rand_nrem = all_dat[x==1, :, :]
        tt.append(ttest_ind(rand_rem, rand_nrem, axis=0, equal_var=False)[0])
        print k

    tt = np.array(tt)

    return tt  

def return_same(rem, nrem, nrand=0):
    return np.array([rem, nrem])

def get_pval_mat(adj):
    nrand = float(adj.shape[0]-1)
    pvals = np.empty([adj.shape[1], adj.shape[2]]); pvals[:] = np.NAN
    for i in range(adj.shape[1]):
        for j in range(adj.shape[2]):
            if adj[0, i, j] == 0: #This should be fixed
                continue
            else:
                pvals[i, j] = np.sum(adj[1:, i, j]>adj[0, i, j])/float(nrand)
                
    return pvals

def get_zscore_mat(adj):
    mu = np.mean(adj[1:, :, :], 0)
    std = np.std(adj[1:, :, :], 0)
    adjz = (adj[0, :, :]-mu)/std
    
    return adjz

def exclude_bad(talPath, bipolar_pairs, monopolar_channels):
    import os
    import numpy as np
    
    bpPairs = []
    mpChans = []
    for talPathi, bipolar_pairsi, monopolar_channelsi in zip(talPath, bipolar_pairs, monopolar_channels):
        gf = open(os.path.dirname(talPathi)+'/good_leads.txt','r')
        goodleads = gf.read().split('\n')
        gf.close
        if os.path.isfile(os.path.dirname(talPathi)+'/bad_leads.txt'):
            bf = open(os.path.dirname(talPathi)+'/bad_leads.txt','r')
            badleads = bf.read().split('\n')
            bf.close
        else:
            badleads = []
        subPairs = np.array([pairi for pairi in bipolar_pairsi if (pairi[0].lstrip('0') in goodleads) and (pairi[1].lstrip('0') in goodleads) and (pairi[0].lstrip('0') not in badleads) and (pairi[1].lstrip('0') not in badleads)])
        subPairs = np.rec.array(subPairs) 
        subChans = np.array([chani for chani in monopolar_channelsi if (chani.lstrip('0') in goodleads) and (chani.lstrip('0') not in badleads)])
        bpPairs.append(subPairs)
        mpChans.append(subChans)
        
        return bpPairs, mpChans

#Main class def
class mne_pipeline():
    
    def get_zscore_power(self, nRandomizations=500):
        from skimage.util import view_as_blocks
        from ptsa.data.filters.MorletWaveletFilter import MorletWaveletFilter
        from scipy.stats import zscore
        from scipy.stats import ttest_ind
        from ptsa.data.TimeSeriesX import TimeSeriesX
        from ptsa.data.events import Events
        from ptsa.data.readers.IndexReader import JsonIndexReader
        reader = JsonIndexReader('/protocols/r1.json')
        
        if self.band=='theta':
            freq=[4., 8.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='lowgamma':
            freq=[30., 60.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 5)
        elif self.band=='highgamma':
            freq=[65., 125.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 10)
        elif self.band=='midgamma':
            freq=[65., 85.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 5)
        elif self.band=='alpha':
            freq=[9., 13.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='beta':
            freq=[16., 28.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='theta_gamma':
            cwtfreq = np.hstack([np.arange(3., 8.+1, 1), np.arange(65., 85.+1, 5)])
        elif self.band=='all_freqs':
            freq = [3., 130.]
            cwtfreq = np.array([3., 4., 5., 6., 7., 8.,
                                9., 10., 11., 12., 13.,
                                16., 18., 20., 22., 24., 26., 28., 
                                30., 32., 34., 36., 38., 40., 42., 44., 46., 48.,
                                50., 55., 60., 65., 70., 75., 80.])

        sub = self.s

        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate
        params['freq'] = freq

        try:
            #Get events and tal data

            #Get available montages
            montages = reader.montages(subject=sub, experiment=self.task)

            #Just use the first montage for the task to avoid confusion
            montage = montages[0]
            self.montage = montage
            sessions = reader.sessions(subject=sub, experiment=self.task, montage=montage)
            
            #Get tal struct
            if self.montage=='0':
                tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
            else:
                tal_path = '/data/eeg/'+sub+'_'+str(self.montage)+'/tal/'+sub+'_'+str(self.montage)+'_talLocs_database_bipol.mat'
            tal_reader = TalReader(filename=tal_path)
            monopolar_channels = tal_reader.get_monopolar_channels()
            bipolar_pairs = tal_reader.get_bipolar_pairs()

            #Get events
            evfiles = list(reader.aggregate_values('task_events', subject=sub, experiment=self.task, montage=montage))

            evs = np.array([])
            for ef in evfiles:
                base_e_reader = BaseEventReader(filename=ef, eliminate_events_with_no_eeg=True)
                base_events = base_e_reader.read()
                if len(evs) == 0:
                    if 'PAL' in self.task:
                        evs = base_events[base_events.type=='STUDY_PAIR']
                    else:
                        evs = base_events[base_events.type=='WORD']
                else:
                    evs = np.concatenate((evs, base_events[base_events.type=='WORD']), axis=0)
            evs = Events(evs)

            #Get EEGs
            eeg_reader = EEGReader(events=evs, channels=monopolar_channels,
                                  start_time=self.win_st, end_time=self.win_fin, buffer_time=1.0)
            base_eegs = eeg_reader.read()
        
        except:
            return
        
        #Get bipolar timeseries
        m2b = MonopolarToBipolarMapper(time_series=base_eegs, bipolar_pairs=bipolar_pairs)
        bp_eegs = m2b.filter()

        print 'Data Loaded.'

        #Only use MTL electrodes
        MTLelecs = [elecidx for elecidx, l in enumerate(tal_reader.tal_struct_array.locTag) if l in MTL_labels]

        if len(MTLelecs) < 2:
            print 'Less than 2 MTL electrodes!'
            return

        bp_eegs = bp_eegs[MTLelecs, :, :]
        MTL_info = tal_reader.tal_struct_array[MTLelecs]
        np.save(self.root+''+sub+'/MTL_info.npy', MTL_info)

        if sub[0:2] == 'FR':
            freq_range = [48., 52.]
        else:
            freq_range = [58., 62.]
        b_filter = ButterworthFilter(time_series=bp_eegs, freq_range=freq_range, filt_type='stop', order=4)
        bp_eegs_filtered = b_filter.filter()
        bp_eegs_filtered = bp_eegs_filtered.resampled(self.samplerate) #downsample
        bp_eegs_filtered['samplerate'] = float(self.samplerate) #for some reason we have to do this.

        wf = MorletWaveletFilter(time_series=bp_eegs_filtered,
                                 freqs=cwtfreq, width=6, output='power')
        pow_wavelet, _ = wf.filter()
        print 'Wavelets processed.'
        np.log10(pow_wavelet.data, out=pow_wavelet.data) #take log for low-power decay at high freqs

        pow_wavelet = pow_wavelet.transpose("events","bipolar_pairs","frequency","time") #transpose for easier computations

        #except:            
        #    return
        
        #z-score by sessions 
        for s in np.unique(evs.session):
            #z-score powers by session
            vals = pow_wavelet[np.array(bp_eegs['events'])['session']==s, :, :, :]
            valsz = zscore(vals, axis=0, ddof=1)
            pow_wavelet[np.array(bp_eegs['events'])['session']==s, :, :, :] = valsz
        
        #Average into time windows -- 200ms for now
        binsize=int(self.samplerate/5.0)
        total_wins = pow_wavelet.shape[3]/binsize
        pow_wavelet = np.array(pow_wavelet) #TimeSeriesX things get in the way
        
        #Reshape into 200ms windows
        powrs = np.reshape(pow_wavelet[:, :, :, :binsize*total_wins],
                           (pow_wavelet.shape[0], pow_wavelet.shape[1], pow_wavelet.shape[2], total_wins, binsize))
        pow_avg = np.mean(powrs, axis=4)

        #Get true t-stat and shuffled t-stats
        from copy import copy
        if 'PAL' in self.task:
            ttrue, _ = ttest_ind(pow_avg[np.array(bp_eegs['events'])['correct']==1, :, :, :], pow_avg[np.array(bp_eegs['events'])['correct']==0, :, :, :], axis=0, equal_var=False)
        else:
            ttrue, _ = ttest_ind(pow_avg[np.array(bp_eegs['events'])['recalled']==1, :, :, :], pow_avg[np.array(bp_eegs['events'])['recalled']==0, :, :, :], axis=0, equal_var=False)
        tstats = []; tstats.append(ttrue)
        if 'PAL' in self.task:
            evsRec = copy(np.array(bp_eegs['events'])['correct'])
        else:
            evsRec = copy(np.array(bp_eegs['events'])['recalled'])
        
        for i in range(nRandomizations):
            np.random.shuffle(evsRec)
            tn, _ = ttest_ind(pow_avg[evsRec==1, :, :, :], pow_avg[evsRec==0, :, :, :], axis=0, equal_var=False)
            tstats.append(tn)
        
        self.tstats = np.array(tstats)
        print 'T-stats processed'
        
        import os
        fn = self.root+'/'+sub+'/'+sub+'_powers_MTL_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        self.dat_file = fn

        try:
            np.save(fn, np.array(tstats))
        except:
            os.mkdir(self.root+'/'+sub+'/')
            np.save(fn, np.array(tstats))
            
    def get_zscore_power_recall(self, nRandomizations=500):
        from skimage.util import view_as_blocks
        from ptsa.data.filters.MorletWaveletFilter import MorletWaveletFilter
        from scipy.stats import zscore
        from scipy.stats import ttest_ind
        from ptsa.data.TimeSeriesX import TimeSeriesX
        
        if self.band=='theta':
            freq=[3., 8.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='lowgamma':
            freq=[30., 60.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 5)
        elif self.band=='highgamma':
            freq=[90., 130.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 10)
        elif self.band=='midgamma':
            freq=[65., 85.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 5)
        elif self.band=='alpha':
            freq=[9., 13.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='beta':
            freq=[16., 28.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='theta_gamma':
            freq=[3., 85.]
            cwtfreq = np.hstack([np.arange(3., 8.+1, 1), np.arange(65., 85.+1, 5)])
        elif self.band=='custom_gamma':
            freq=[45., 100.]
            cwtfreq = np.array([45., 50., 55., 60., 90., 100.])
        elif self.band=='all_freqs':
            freq = [3., 130.]
            cwtfreq = np.array([3, 4, 5, 6, 7, 8, 
                                9, 10, 11, 12, 13, 
                                16, 18, 20, 22, 24, 26, 28, 
                                30, 35, 40, 45, 50, 55, 60, 
                                90, 100, 110, 120, 130])

        sub = self.s

        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate
        params['freq'] = freq

        try:
        
            #Get tal struct
            tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
            tal_reader = TalReader(filename=tal_path)
            monopolar_channels = tal_reader.get_monopolar_channels()
            bipolar_pairs = tal_reader.get_bipolar_pairs()

            #Get events
            e_path = '/data/events/'+self.task+'/'+sub+'_events.mat'
            base_e_reader = BaseEventReader(filename=e_path, eliminate_events_with_no_eeg=True)
            base_events = base_e_reader.read()
            if self.task=='pyFR':
                rectime=45
            else:
                rectime=30
            evs = create_baseline_events(base_events, 3*1000, rectime*1000)
            evs = evs[np.logical_or(evs.type=='REC_BASE', np.logical_and(evs.type=='REC_WORD', evs.intrusion==0))]

            #Get EEGs
            eeg_reader_rec = EEGReader(events=evs[evs.type=='REC_WORD'], channels=monopolar_channels,
                                  start_time=-0.5, end_time=0.0, buffer_time=1.0)
            base_eegs_rec = eeg_reader_rec.read()

            eeg_reader_base = EEGReader(events=evs[evs.type=='REC_BASE'], channels=monopolar_channels,
                                  start_time=0.0, end_time=0.5, buffer_time=1.0)
            base_eegs_base = eeg_reader_base.read()
        except:
            return
            
        def process_bipolar_pows(base_eegs, bipolar_pairs):

            #Get bipolar timeseries
            m2b = MonopolarToBipolarMapper(time_series=base_eegs, bipolar_pairs=bipolar_pairs)
            bp_eegs = m2b.filter()

            if sub[0:2] == 'FR':
                freq_range = [48., 52.]
            else:
                freq_range = [58., 62.]
            b_filter = ButterworthFilter(time_series=bp_eegs, freq_range=freq_range, filt_type='stop', order=4)
            bp_eegs_filtered = b_filter.filter()
            bp_eegs_filtered = TimeSeriesX(bp_eegs_filtered.resampled(self.samplerate)) #downsample
            bp_eegs_filtered['samplerate'] = float(self.samplerate) #for some reason we have to do this.

            wf = MorletWaveletFilter(time_series=bp_eegs_filtered,
                                     freqs=cwtfreq, width=6, output='power')
            pow_wavelet, _ = wf.filter()
            np.log10(pow_wavelet.data, out=pow_wavelet.data) #take log for low-power decay at high freqs
            pow_wavelet = pow_wavelet.transpose("events","bipolar_pairs","frequency","time") #transpose for easier computations
            pow_wavelet = np.array(pow_wavelet)
            return pow_wavelet

        
        pow_rec = process_bipolar_pows(base_eegs_rec, bipolar_pairs)
        pow_base = process_bipolar_pows(base_eegs_base, bipolar_pairs)
        pow_wavelet = np.concatenate((pow_rec, pow_base), axis=0)
        rec_evs = np.hstack((np.ones(pow_rec.shape[0]), np.zeros(pow_base.shape[0])))
        
        #z-score by sessions 
        for s in np.unique(evs.session):
            #z-score powers by session
            vals = pow_wavelet[evs.session==s, :, :, :]
            valsz = zscore(vals, axis=0, ddof=1)
            pow_wavelet[evs.session==s, :, :, :] = valsz
        
        #Average into time windows -- 200ms for now
        binsize=int(self.samplerate/4.0)
        total_wins = pow_wavelet.shape[3]/binsize
        pow_wavelet = np.array(pow_wavelet) #TimeSeriesX things get in the way
        
        #Reshape into 200ms windows
        powrs = np.reshape(pow_wavelet[:, :, :, :binsize*total_wins],
                           (pow_wavelet.shape[0], pow_wavelet.shape[1], pow_wavelet.shape[2], total_wins, binsize))
        pow_avg = np.mean(powrs, axis=4)

        #Get true t-stat and shuffled t-stats
        from copy import copy
        ttrue, _ = ttest_ind(pow_avg[rec_evs==1, :, :, :], pow_avg[rec_evs==0, :, :, :], axis=0, equal_var=False)
        tstats = []; tstats.append(ttrue)
        evsRec = copy(rec_evs)
        
        for i in range(nRandomizations):
            np.random.shuffle(evsRec)
            tn, _ = ttest_ind(pow_avg[evsRec==1, :, :, :], pow_avg[evsRec==0, :, :, :], axis=0, equal_var=False)
            tstats.append(tn)
        
        self.tstats = np.array(tstats)
        
        import os
        fn = self.root+'/'+sub+'/'+sub+'_powers_recall_'+self.band+'.npy'
        self.dat_file = fn

        try:
            np.save(fn, np.array(tstats))
        except:
            os.mkdir('/scratch/esolo/mne_analysis/'+sub+'/')
            np.save(fn, np.array(tstats))
            
    def compute_phases_recall(self):
        from skimage.util import view_as_blocks
        from ptsa.data.filters.MorletWaveletFilter import MorletWaveletFilter
        from scipy.stats import zscore
        from scipy.stats import ttest_ind
        from ptsa.data.TimeSeriesX import TimeSeriesX
        import pycircstat as circ
        from glob import glob
        from copy import copy

        if self.band=='theta':
            freq=[3., 8.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='lowgamma':
            freq=[30., 60.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 5)
        elif self.band=='highgamma':
            freq=[90., 130.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 10)
        elif self.band=='midgamma':
            freq=[65., 85.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 5)
        elif self.band=='alpha':
            freq=[9., 13.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='beta':
            freq=[16., 28.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='theta_gamma':
            cwtfreq = np.hstack([np.arange(3., 8.+1, 1), np.arange(65., 85.+1, 5)])
        elif self.band=='custom_gamma':
            cwtfreq = np.array([45., 50., 55., 60., 90., 100.])
        else:
            raise ValueError

        sub = self.s
        
        fn = self.root+'/'+sub+'/'+sub+'_manual_plv_recall_'+self.band+'.npy'
        if len(glob(fn)) > 0: #See if we've done this combo already
            print 'Already processed!'
            return
        fn = self.root+'/'+sub+'/'+sub+'_phases_recall_'+self.band+'.npy'
        if len(glob(fn)) > 0:
            print 'Already processed!'
            return

        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate

        #Get tal struct
        tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
        tal_reader = TalReader(filename=tal_path)
        monopolar_channels = tal_reader.get_monopolar_channels()
        bipolar_pairs = tal_reader.get_bipolar_pairs()
        sub_elecs = tal_reader.tal_struct_array      

        #Get events
        e_path = '/data/events/'+self.task+'/'+sub+'_events.mat'
        base_e_reader = BaseEventReader(filename=e_path, eliminate_events_with_no_eeg=True)
        base_events = base_e_reader.read()
        if self.task=='pyFR':
            rectime=45
        else:
            rectime=30
        evs = create_baseline_events(base_events, 3*1000, rectime*1000)
        evs = evs[np.logical_or(evs.type=='REC_BASE', np.logical_and(evs.type=='REC_WORD', evs.intrusion==0))]

        #Get EEGs
        eeg_reader_rec = EEGReader(events=evs[evs.type=='REC_WORD'], channels=monopolar_channels,
                              start_time=-0.5, end_time=0.0, buffer_time=1.0)
        base_eegs_rec = eeg_reader_rec.read()

        eeg_reader_base = EEGReader(events=evs[evs.type=='REC_BASE'], channels=monopolar_channels,
                              start_time=0.0, end_time=0.5, buffer_time=1.0)
        base_eegs_base = eeg_reader_base.read()

        def process_bipolar_phases(base_eegs, bipolar_pairs):

            #Get bipolar timeseries
            m2b = MonopolarToBipolarMapper(time_series=base_eegs, bipolar_pairs=bipolar_pairs)
            bp_eegs = m2b.filter()

            if sub[0:2] == 'FR':
                freq_range = [48., 52.]
            else:
                freq_range = [58., 62.]
            b_filter = ButterworthFilter(time_series=bp_eegs, freq_range=freq_range, filt_type='stop', order=4)
            bp_eegs_filtered = b_filter.filter()
            bp_eegs_filtered = TimeSeriesX(bp_eegs_filtered.resampled(self.samplerate)) #downsample
            bp_eegs_filtered['samplerate'] = float(self.samplerate) #for some reason we have to do this.

            wf = MorletWaveletFilter(time_series=bp_eegs_filtered,
                                     freqs=cwtfreq, width=6, output='phase')
            _, phase_wavelet = wf.filter()
            phase_wavelet = phase_wavelet.transpose("events","bipolar_pairs","frequency","time") #transpose for easier computations
            phase_wavelet = np.array(phase_wavelet)
            return phase_wavelet

        phase_rec = process_bipolar_phases(base_eegs_rec, bipolar_pairs)
        phase_base = process_bipolar_phases(base_eegs_base, bipolar_pairs)
        phase_wavelet = np.concatenate((phase_rec, phase_base), axis=0)
        rec_evs = np.hstack((np.ones(phase_rec.shape[0]), np.zeros(phase_base.shape[0])))

        print 'Wavelets complete.'

        import os
        try:
            np.save(self.root+'/'+sub+'/'+sub+'_phases_recall_'+self.band+'.npy', phase_wavelet)
            np.save(self.root+'/'+sub+'/'+sub+'_events_recall.npy', rec_evs)
        except:
            os.mkdir(self.root+'/'+sub+'/')
            np.save(self.root+'/'+sub+'/'+sub+'_phases_recall_'+self.band+'.npy', phase_wavelet)
            np.save(self.root+'/'+sub+'/'+sub+'_events_recall.npy', rec_evs)
        
        return
    
    def manual_plv_recall(self, nRandomizations=500):
        
        from skimage.util import view_as_blocks
        from ptsa.data.filters.MorletWaveletFilter import MorletWaveletFilter
        from scipy.stats import zscore
        from scipy.stats import ttest_ind
        from ptsa.data.TimeSeriesX import TimeSeriesX
        import pycircstat as circ
        from glob import glob
        from copy import copy
        import os.path
        
        sub = self.s
        
        #Abort if the phases haven't been calculated
        try: 
            phase_wavelet = np.load(self.root+'/'+sub+'/'+sub+'_phases_recall_'+self.band+'.npy')
        except:
            return
        
        self.root = '/scratch/esolo/mne_analysis/'
        
        fn = self.root+'/'+sub+'/'+sub+'_manual_plv_recall_'+self.band+'.npy'
        if len(glob(fn)) > 0: #See if we've done this combo already
            print 'Already processed!'
            return
        
        try:
            #Get tal struct
            tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
            tal_reader = TalReader(filename=tal_path)
            monopolar_channels = tal_reader.get_monopolar_channels()
            bipolar_pairs = tal_reader.get_bipolar_pairs()
            sub_elecs = tal_reader.tal_struct_array  

            #Get events
            evs = np.load(self.root+'/'+sub+'/'+sub+'_events_recall.npy')
        except:
            return
                
        #For time averaging
        binsize=int(self.samplerate/4.0)
        total_wins = phase_wavelet.shape[3]/binsize
        
        #Get phase differences and put into matrix
        import itertools
        combs = list(itertools.combinations(range(0, len(bipolar_pairs)), 2))            

        pairs_to_skip = [] #Find bipolars with shared monopolar contact
        for c in combs:
            t = list(sub_elecs[c[0]]['channel'])
            t.extend(sub_elecs[c[1]]['channel'])
            if len(np.unique(t))<4:
                pairs_to_skip.append(c)
        
        #Load preshuffled vectors
        try:
            shuffled_vecs = np.load('/scratch/esolo/mne_analysis/'+sub+'/shuffled_vecs_recall.npy')
        except:
            shuffled_vecs = []
            all_idxs = np.arange(len(evs))
                
            for k in range(nRandomizations):
                np.random.shuffle(all_idxs)
                shuffled_vecs.append(copy(all_idxs))
                
            shuffled_vecs = np.array(shuffled_vecs)
            np.save('/scratch/esolo/mne_analysis/'+sub+'/shuffled_vecs_recall.npy', shuffled_vecs)
        
        for idx_counter, c in enumerate(combs):
            elec1 = c[0]
            elec2 = c[1]
            
            fn = self.root+'/'+sub+'/'+sub+'_manual_plv_recall_'+self.band+'/'+str(elec1)+'_'+str(elec2)+'.npy'
            if os.path.isfile(fn):
                continue
            
            if c in pairs_to_skip:  #Skip pairs with shared monopolar
                continue
                            
            diff = phase_wavelet[:, elec1, :, :binsize*total_wins]-phase_wavelet[:, elec2, :, :binsize*total_wins] #circ.mean will take care of the differences
            diffrs = np.reshape(diff, (diff.shape[0], diff.shape[1], total_wins, binsize))
            phase_avg = circ.mean(diffrs, axis=3)
                            
            #print str(idx_counter)+'/'+str(len(combs))
                        
            #Get true t-stat and shuffled t-stats
            dtrue = np.squeeze(circ.resultant_vector_length(phase_avg[evs==1, :, :], axis=0) - circ.resultant_vector_length(phase_avg[evs==0, :, :], axis=0))
            dstats = []; dstats.append(dtrue)
            evsRec = copy(evs)
                
            for i in range(nRandomizations):
                evs_shuf = evsRec[shuffled_vecs[i, :]]
                dn = np.squeeze(circ.resultant_vector_length(phase_avg[evs_shuf==1, :, :], axis=0) - circ.resultant_vector_length(phase_avg[evs_shuf==0, :, :], axis=0))
                dstats.append(dn)
                #print str(i)+'/'+str(nRandomizations)
            
            print 'Trial shuffling complete.'
        
            self.dstats = np.array(dstats)
        
            import os
            self.dat_file = fn

            try:
                np.save(fn, np.array(dstats))
            except:
                os.mkdir(self.root+'/'+sub+'/'+sub+'_manual_plv_recall_'+self.band+'/')
                np.save(fn, np.array(dstats))
                
    def pool_plv_recall(self, nRandomizations=500):
        from glob import glob

        sub = self.s
        
        fn = self.root+'/'+sub+'/'+sub+'_manual_plv_recall_'+self.band+'.npy'
        if len(glob(fn)) > 0: #See if we've done this combo already
            print 'Already processed!'
            return
        
        try:
            #Get tal struct
            tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
            tal_reader = TalReader(filename=tal_path)
            monopolar_channels = tal_reader.get_monopolar_channels()
            bipolar_pairs = tal_reader.get_bipolar_pairs()
            sub_elecs = tal_reader.tal_struct_array  
        except:
            return
        
        import itertools
        combs = list(itertools.combinations(range(0, len(bipolar_pairs)), 2))            

        pairs_to_skip = [] #Find bipolars with shared monopolar contact
        for c in combs:
            t = list(sub_elecs[c[0]]['channel'])
            t.extend(sub_elecs[c[1]]['channel'])
            if len(np.unique(t))<4:
                pairs_to_skip.append(c)
        
        total_pairs = len(combs)-len(pairs_to_skip)
        
        try:
            dir_ = self.root+'/'+sub+'/'+sub+'_manual_plv_recall_'+self.band+'/*npy'
            if len(glob(dir_)) == total_pairs:
                print 'All pairs processed! Moving on....'
                pass
            else:
                return
        except:
            return
        
        #Get a sample item
        try:
            tmp = np.load(glob(dir_)[0])
        except:
            return
        numfreqs = tmp.shape[1]
        numepochs = tmp.shape[2]
        
        #Make adjacency matrix
        mat = np.empty([nRandomizations+1, len(sub_elecs), len(sub_elecs), numfreqs, numepochs])
        mat[:] = np.nan
        
        #Add each comparison to the adjacency matrix
        for f in glob(dir_):
            try:
                D = np.load(f)
            except:
                continue
            elec1 = int(f.split('/')[-1].split('.')[0].split('_')[0])
            elec2 = int(f.split('/')[-1].split('.')[0].split('_')[1])
            mat[:, elec1, elec2, :, :]  = D

        np.save(fn, mat)     
        return
            
    def compute_phases(self, MTLonly=False):
        from skimage.util import view_as_blocks
        from ptsa.data.filters.MorletWaveletFilter import MorletWaveletFilter
        from scipy.stats import zscore
        from scipy.stats import ttest_ind
        from ptsa.data.TimeSeriesX import TimeSeriesX
        import pycircstat as circ
        from glob import glob
        from copy import copy
        from ptsa.data.events import Events
        from ptsa.data.readers.IndexReader import JsonIndexReader
        reader = JsonIndexReader('/protocols/r1.json')
        
        MTL_labels = ['Left CA1', 'Left CA2', 'Left CA3', 'Left DG', 'Left Sub', 'Left PRC', 'Left EC', 'Left PHC',
         'Right CA1', 'Right CA2', 'Right CA3', 'Right DG', 'Right Sub', 'Right PRC', 'Right EC', 'Right PHC']

        if self.band=='theta':
            freq=[3., 8.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
#         elif self.band=='lowgamma':
#             freq=[30., 60.]
#             cwtfreq = np.arange(freq[0], freq[1]+1, 5)
        elif self.band=='lowgamma':
            freq=[32., 48.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='highgamma':
            freq=[90., 130.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 10)
        elif self.band=='midgamma':
            freq=[65., 85.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 5)
        elif self.band=='alpha':
            freq=[9., 13.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='beta':
            freq=[16., 28.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='all_freqs':
            freq = [3., 130.]
            cwtfreq = np.array([3, 4, 5, 6, 7, 8, 
                                9, 10, 11, 12, 13, 
                                16, 18, 20, 22, 24, 26, 28, 
                                30, 35, 40, 45, 50, 55, 60, 
                                90, 100, 110, 120, 130])
        else:
            raise ValueError

        sub = self.s
        
#         fn = self.root+'/'+sub+'/'+sub+'_manual_plv_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
#         if len(glob(fn)) > 0: #See if we've done this combo already
#             print 'Already processed!'
#             return
#         fn = self.root+'/'+sub+'/'+sub+'_phases_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
#         if len(glob(fn)) > 0:
#             print 'Already processed!'
#             return
        

#        try:
        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate
        params['freq'] = freq

        #Get tal struct
        tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
        tal_reader = TalReader(filename=tal_path)
        monopolar_channels = tal_reader.get_monopolar_channels()
        bipolar_pairs = tal_reader.get_bipolar_pairs()
        sub_elecs = tal_reader.tal_struct_array

        #Get available montages
        montages = reader.montages(subject=sub, experiment=self.task)

        #Just use the first montage for the task to avoid confusion
        montage=montages[0]
        sessions = reader.sessions(subject=sub, experiment=self.task, montage=montage)

        #Get events
        evfiles = list(reader.aggregate_values('task_events', subject=sub, experiment=self.task, montage=montage))

        evs = np.array([])
        for ef in evfiles:
            base_e_reader = BaseEventReader(filename=ef, eliminate_events_with_no_eeg=True)
            base_events = base_e_reader.read()
            if len(evs) == 0:
                evs = base_events[base_events.type=='WORD']
            else:
                evs = np.concatenate((evs, base_events[base_events.type=='WORD']), axis=0)
        evs = Events(evs)
        self.evs = evs

        #Get EEGs
        eeg_reader = EEGReader(events=evs, channels=monopolar_channels,
                              start_time=self.win_st, end_time=self.win_fin, buffer_time=1.0)
        base_eegs = eeg_reader.read()

        #Get bipolar timeseries
        m2b = MonopolarToBipolarMapper(time_series=base_eegs, bipolar_pairs=bipolar_pairs)
        bp_eegs = m2b.filter()

        if MTLonly:
            #Only use MTL electrodes
            MTLelecs = [elecidx for elecidx, l in enumerate(tal_reader.tal_struct_array.locTag) if l in MTL_labels]

            if len(MTLelecs) < 2:
                print 'Less than 2 MTL electrodes!'
                return

            bp_eegs = bp_eegs[MTLelecs, :, :]
            MTL_info = tal_reader.tal_struct_array[MTLelecs]
            np.save(self.root+''+sub+'/MTL_info.npy', MTL_info)

        print 'Data loaded.'

        if sub[0:2] == 'FR':
            freq_range = [48., 52.]
        else:
            freq_range = [58., 62.]
        b_filter = ButterworthFilter(time_series=bp_eegs, freq_range=freq_range, filt_type='stop', order=4)
        bp_eegs_filtered = b_filter.filter()
        bp_eegs_filtered = TimeSeriesX(bp_eegs_filtered.resampled(self.samplerate)) #downsample
        bp_eegs_filtered['samplerate'] = float(self.samplerate) #for some reason we have to do this.

        wf = MorletWaveletFilter(time_series=bp_eegs_filtered,
                                 freqs=cwtfreq, width=6, output='phase')
        _, phase_wavelet = wf.filter()
        phase_wavelet = phase_wavelet.transpose("events","bipolar_pairs","frequency","time") #transpose for easier computations
        phase_wavelet = np.array(phase_wavelet) #TimeSeriesX things get in the way

        print 'Wavelets complete.'

        fn = self.root+'/'+sub+'/'+sub+'_phases_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        if MTLonly:
            fn = self.root+'/'+sub+'/'+sub+'_phases_MTL_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'

        import os
        try:
            np.save(fn, phase_wavelet)
        except:
            os.mkdir(self.root+'/'+sub+'/')
            np.save(fn, phase_wavelet)
            
#        except:
#            return
        
        return
    
    def compute_phases_evoked(self):
        from skimage.util import view_as_blocks
        from ptsa.data.filters.MorletWaveletFilter import MorletWaveletFilter
        from scipy.stats import zscore
        from scipy.stats import ttest_ind
        from ptsa.data.TimeSeriesX import TimeSeriesX
        import pycircstat as circ
        from glob import glob
        from copy import copy

        if self.band=='theta':
            freq=[4., 9.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='alpha':
            freq=[10.,13.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='theta_alpha':
            freq=[4.,13.]
            cwtfreq = np.array([4., 6., 8., 10., 12.])
        else:
            raise ValueError

        sub = self.s
        
        fn = self.root+'/'+sub+'/'+sub+'_manual_plv_ER2_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        if len(glob(fn)) > 0: #See if we've done this combo already
            print 'Already processed!'
            return
        fn = self.root+'/'+sub+'/'+sub+'_phases_ER2_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        if len(glob(fn)) > 0:
            print 'Already processed!'
            return
        

        try:
            #Set up some params
            params = {}
            params['window_start'] = self.win_st
            params['window_finish'] = self.win_fin
            params['samplerate'] = self.samplerate
            params['freq'] = freq

            #Get tal struct
            tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
            tal_reader = TalReader(filename=tal_path)
            monopolar_channels = tal_reader.get_monopolar_channels()
            bipolar_pairs = tal_reader.get_bipolar_pairs()
            sub_elecs = tal_reader.tal_struct_array      

            #Get events
            e_path = '/data/events/'+self.task+'/'+sub+'_events.mat'
            base_e_reader = BaseEventReader(filename=e_path, eliminate_events_with_no_eeg=True)
            base_events = base_e_reader.read()
            evs = base_events[base_events.type=='WORD']
            self.evs = evs

            #Get EEGs
            eeg_reader = EEGReader(events=evs, channels=monopolar_channels,
                                  start_time=self.win_st, end_time=self.win_fin, buffer_time=1.0)
            base_eegs = eeg_reader.read()

            #Get bipolar timeseries
            m2b = MonopolarToBipolarMapper(time_series=base_eegs, bipolar_pairs=bipolar_pairs)
            bp_eegs = m2b.filter()

            print 'Data loaded.'

            if sub[0:2] == 'FR':
                freq_range = [48., 52.]
            else:
                freq_range = [58., 62.]
            b_filter = ButterworthFilter(time_series=bp_eegs, freq_range=freq_range, filt_type='stop', order=4)
            bp_eegs_filtered = b_filter.filter()
            bp_eegs_filtered = TimeSeriesX(bp_eegs_filtered.resampled(self.samplerate)) #downsample
            bp_eegs_filtered['samplerate'] = float(self.samplerate) #for some reason we have to do this.
            bp_eegs_filtered = bp_eegs_filtered.transpose('time', 'events', 'bipolar_pairs')

            for sess in np.unique(evs.session): #For each session
                
                print sess

                #Subtract the baseline from each event
                sessdata = bp_eegs_filtered[:, evs.session==sess, :]
                baseline = sessdata[400:500, :, :].mean('time') #Take -200ms baseline
                baseline_tile = np.tile(baseline, (sessdata.shape[0], 1, 1))
                bp_eegs_filtered[:, evs.session==sess, :] = bp_eegs_filtered[:, evs.session==sess, :]-baseline_tile

                #Subtract average trace for all remembered and all forgotten trials separately
                #OR
                #Don't group things
                evfilter = evs.session==sess
                evmean = bp_eegs_filtered[:, evfilter, :].mean('events')
                for i in range(evmean.shape[1]): #For each electrode
                    avgtrace = evmean[:, i]
                    tmp = np.tile(avgtrace, (np.sum(evfilter), 1)).T
                    bp_eegs_filtered[:, evfilter, i] = bp_eegs_filtered[:, evfilter, i]-tmp
                    print i
                    
                #Do rem first
#                 evfilter = np.logical_and(evs.session==sess, evs.recalled==1)
#                 evmean = bp_eegs_filtered[:, evfilter, :].mean('events')
#                 for i in range(evmean.shape[1]): #For each electrode
#                     avgtrace = evmean[:, i]
#                     tmp = np.tile(avgtrace, (np.sum(evfilter), 1)).T
#                     bp_eegs_filtered[:, evfilter, i] = bp_eegs_filtered[:, evfilter, i]-tmp
#                     print i

#                 #Now not-rem
#                 evfilter = np.logical_and(evs.session==sess, evs.recalled==0)
#                 evmean = bp_eegs_filtered[:, evfilter, :].mean('events')
#                 for i in range(evmean.shape[1]): #For each electrode
#                     avgtrace = evmean[:, i]
#                     tmp = np.tile(avgtrace, (np.sum(evfilter), 1)).T
#                     bp_eegs_filtered[:, evfilter, i] = bp_eegs_filtered[:, evfilter, i]-tmp

            bp_eegs_filtered = bp_eegs_filtered.transpose('bipolar_pairs', 'events', 'time')
            wf = MorletWaveletFilter(time_series=bp_eegs_filtered,
                                     freqs=cwtfreq, width=6, output='phase')
            _, phase_wavelet = wf.filter()
            phase_wavelet = phase_wavelet.transpose("events","bipolar_pairs","frequency","time") #transpose for easier computations
            phase_wavelet = np.array(phase_wavelet) #TimeSeriesX things get in the way

            print 'Wavelets complete.'

            import os
            try:
                np.save(self.root+'/'+sub+'/'+sub+'_phases_ER2_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy', phase_wavelet)
            except:
                os.mkdir(self.root+'/'+sub+'/')
                np.save(self.root+'/'+sub+'/'+sub+'_phases_ER2_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy', phase_wavelet)
            
        except:
            return
        
        return
    
    def manual_plv(self, nRandomizations=500):
        
        from skimage.util import view_as_blocks
        from ptsa.data.filters.MorletWaveletFilter import MorletWaveletFilter
        from scipy.stats import zscore
        from scipy.stats import ttest_ind
        from ptsa.data.TimeSeriesX import TimeSeriesX
        import pycircstat as circ
        from glob import glob
        from copy import copy
        import os.path
        
        sub = self.s
        
        #Abort if the phases haven't been calculated
        try: 
            phase_wavelet = np.load(self.root+'/'+sub+'/'+sub+'_phases_ER2_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy')
        except:
            return
        
        self.root = '/scratch/esolo/mne_analysis/'
        
        fn = self.root+'/'+sub+'/'+sub+'_manual_plv_ER2_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        if len(glob(fn)) > 0: #See if we've done this combo already
            print 'Already processed!'
            return
        
        try:
            #Get tal struct
            tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
            tal_reader = TalReader(filename=tal_path)
            monopolar_channels = tal_reader.get_monopolar_channels()
            bipolar_pairs = tal_reader.get_bipolar_pairs()
            sub_elecs = tal_reader.tal_struct_array  

            #Get events
            e_path = '/data/events/'+self.task+'/'+sub+'_events.mat'
            base_e_reader = BaseEventReader(filename=e_path, eliminate_events_with_no_eeg=True)
            base_events = base_e_reader.read()
            evs = base_events[base_events.type=='WORD']
            self.evs = evs
        except:
            return
                
        #For time averaging
        binsize=int(self.samplerate/5.0)
        total_wins = phase_wavelet.shape[3]/binsize
        
        #Get phase differences and put into matrix
        import itertools
        combs = list(itertools.combinations(range(0, len(bipolar_pairs)), 2))            

        pairs_to_skip = [] #Find bipolars with shared monopolar contact
        for c in combs:
            t = list(sub_elecs[c[0]]['channel'])
            t.extend(sub_elecs[c[1]]['channel'])
            if len(np.unique(t))<4:
                pairs_to_skip.append(c)
        
        #Load preshuffled vectors
        try:
            shuffled_vecs = np.load('/scratch/esolo/mne_analysis/'+sub+'/shuffled_vecs.npy')
        except:
            shuffled_vecs = []
            all_idxs = np.arange(len(evs))
                
            for k in range(nRandomizations):
                np.random.shuffle(all_idxs)
                shuffled_vecs.append(copy(all_idxs))
                
            shuffled_vecs = np.array(shuffled_vecs)
            np.save('/scratch/esolo/mne_analysis/'+sub+'/shuffled_vecs.npy', shuffled_vecs)
        
        
        for idx_counter, c in enumerate(combs):
            elec1 = c[0]
            elec2 = c[1]
            
            fn = self.root+'/'+sub+'/'+sub+'_manual_plv_ER2_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'/'+str(elec1)+'_'+str(elec2)+'.npy'
            if os.path.isfile(fn):
                continue
            
            if c in pairs_to_skip:  #Skip pairs with shared monopolar
                continue
                            
            diff = phase_wavelet[:, elec1, :, :binsize*total_wins]-phase_wavelet[:, elec2, :, :binsize*total_wins]
            diffrs = np.reshape(diff, (diff.shape[0], diff.shape[1], total_wins, binsize))
            phase_avg = circ.mean(diffrs, axis=3)
                            
            #print str(idx_counter)+'/'+str(len(combs))
                        
            #Get true t-stat and shuffled t-stats
            dtrue = np.squeeze(circ.resultant_vector_length(phase_avg[evs.recalled==1, :, :], axis=0) - circ.resultant_vector_length(phase_avg[evs.recalled==0, :, :], axis=0))
            dstats = []; dstats.append(dtrue)
            evsRec = copy(evs.recalled)
                
            for i in range(nRandomizations):
                evs_shuf = evsRec[shuffled_vecs[i, :]]
                dn = np.squeeze(circ.resultant_vector_length(phase_avg[evs_shuf==1, :, :], axis=0) - circ.resultant_vector_length(phase_avg[evs_shuf==0, :, :], axis=0))
                dstats.append(dn)
                #print str(i)+'/'+str(nRandomizations)
            
            print 'Trial shuffling complete.'
        
            self.dstats = np.array(dstats)
        
            import os
            self.dat_file = fn

            try:
                np.save(fn, np.array(dstats))
            except:
                os.mkdir(self.root+'/'+sub+'/'+sub+'_manual_plv_ER2_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'/')
                np.save(fn, np.array(dstats))
    
    def pool_plv(self, nRandomizations=500):
        from glob import glob

        sub = self.s
        
        fn = self.root+'/'+sub+'/'+sub+'_manual_plv_ER2_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        if len(glob(fn)) > 0: #See if we've done this combo already
            print 'Already processed!'
            return
        
        try:
            #Get tal struct
            tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
            tal_reader = TalReader(filename=tal_path)
            monopolar_channels = tal_reader.get_monopolar_channels()
            bipolar_pairs = tal_reader.get_bipolar_pairs()
            sub_elecs = tal_reader.tal_struct_array  
        except:
            return
        
        import itertools
        combs = list(itertools.combinations(range(0, len(bipolar_pairs)), 2))            

        pairs_to_skip = [] #Find bipolars with shared monopolar contact
        for c in combs:
            t = list(sub_elecs[c[0]]['channel'])
            t.extend(sub_elecs[c[1]]['channel'])
            if len(np.unique(t))<4:
                pairs_to_skip.append(c)
        
        total_pairs = len(combs)-len(pairs_to_skip)
        
        try:
            dir_ = self.root+'/'+sub+'/'+sub+'_manual_plv_ER2_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'/*npy'
            if len(glob(dir_)) == total_pairs:
                print 'All pairs processed! Moving on....'
                pass
            else:
                return
        except:
            return
        
        #Get a sample item
        try:
            tmp = np.load(glob(dir_)[0])
        except:
            return
        numfreqs = tmp.shape[1]
        numepochs = tmp.shape[2]
        
        #Make adjacency matrix
        mat = np.empty([nRandomizations+1, len(sub_elecs), len(sub_elecs), numfreqs, numepochs])
        mat[:] = np.nan
        
        #Add each comparison to the adjacency matrix
        for f in glob(dir_):
            try:
                D = np.load(f)
            except:
                continue
            elec1 = int(f.split('/')[-1].split('.')[0].split('_')[0])
            elec2 = int(f.split('/')[-1].split('.')[0].split('_')[1])
            mat[:, elec1, elec2, :, :]  = D

        np.save(fn, mat)     
        return
    
    def return_preshuffled(self, rem, nrem, nRandomizations=0):
        sub = self.s
        D = np.load(self.root+''+sub+'/'+sub+'_mne_'+self.conn_type+'_'+self.mode+'_fstat_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy')
        return D
    
    def __init__(self, s, band, win_st=0.0, win_fin=1.6, task='RAM_FR1', montage=0,
                 samplerate=500.0, conn_type='coh', mode='multitaper',
                root='/scratch/esolo/mne_analysis/'):
        self.s = s
        self.band = band
        self.win_st = win_st
        self.win_fin = win_fin
        self.task = task
        self.samplerate = samplerate
        self.conn_type = conn_type
        self.mode = mode
        self.root = root
        self.montage=montage

    def mne_coh_pipeline(self):
        from ptsa.data.events import Events
        from ptsa.data.readers.IndexReader import JsonIndexReader
        reader = JsonIndexReader('/protocols/r1.json')
        from skimage.util import view_as_blocks
        
        if self.mode=='cwt_morlet':
            if self.band=='theta':
                freq=[3., 8.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 1)
            elif self.band=='beta':
                freq=[15., 25.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='lowgamma':
                freq=[30., 40.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='highgamma':
                freq=[90., 105.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            else:
                raise ValueError
        else:
            if self.band=='theta':
                freq=[4., 8.]
            elif self.band=='alpha':
                freq=[9., 13.]
            elif self.band=='beta':
                freq=[16., 28.]
            elif self.band=='lowgamma':
                freq=[30., 60.]
            elif self.band=='highgamma':
                freq=[65., 125.]
            else:
                raise ValueError

        sub = self.s

        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate
        params['freq'] = freq

        #Get events and tal data
        try:
            #Get tal struct
            if self.montage=='0':
                tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
            else:
                tal_path = '/data/eeg/'+sub+'_'+str(self.montage)+'/tal/'+sub+'_'+str(self.montage)+'_talLocs_database_bipol.mat'
            tal_reader = TalReader(filename=tal_path)
            monopolar_channels = tal_reader.get_monopolar_channels()
            bipolar_pairs = tal_reader.get_bipolar_pairs()

            #Get available montages?
            #montages = reader.montages(subject=sub, experiment=self.task)

            #Just use the first montage for the task to avoid confusion
            montage = self.montage
            sessions = reader.sessions(subject=sub, experiment=self.task, montage=montage)

            old_method=False

            #Get events
            if old_method==True:
                #Get events
                if self.montage=='0':
                    e_path = '/data/events/RAM_'+self.task+'/'+sub+'_events.mat'
                else:
                    e_path = '/data/events/RAM_'+self.task+'/'+sub+'_'+str(self.montage)+'_events.mat'
                base_e_reader = BaseEventReader(filename=e_path, eliminate_events_with_no_eeg=True)
                base_events = base_e_reader.read()
                evs = base_events[base_events.type=='WORD']

            else:
                evfiles = list(reader.aggregate_values('task_events', subject=sub, experiment=self.task, montage=montage))

                evs = np.array([])
                for ef in evfiles:
                    base_e_reader = BaseEventReader(filename=ef, eliminate_events_with_no_eeg=True)
                    base_events = base_e_reader.read()
                    if len(evs) == 0:
                        if 'PAL' in self.task:
                            evs = base_events[base_events.type=='STUDY_PAIR']
                        else:
                            evs = base_events[base_events.type=='WORD']
                    else:
                        evs = np.concatenate((evs, base_events[base_events.type=='WORD']), axis=0)
                evs = Events(evs)

            #Get EEGs
            eeg_reader = EEGReader(events=evs, channels=monopolar_channels,
                                  start_time=self.win_st, end_time=self.win_fin, buffer_time=0.0)
            base_eegs = eeg_reader.read()        
        except:
            return

        #Get bipolar timeseries
        m2b = MonopolarToBipolarMapper(time_series=base_eegs, bipolar_pairs=bipolar_pairs)
        bp_eegs = m2b.filter()

        if sub[0:2] == 'FR':
            freq_range = [48., 52.]
        else:
            freq_range = [58., 62.]
        b_filter = ButterworthFilter(time_series=bp_eegs, freq_range=freq_range, filt_type='stop', order=4)
        bp_eegs_filtered = b_filter.filter()

        #Downsample for speed
        bp_eegs_filtered = bp_eegs_filtered.resampled(self.samplerate)

        #Create MNE dataset
        n_channels = bp_eegs_filtered.shape[0]
        ch_names = list(tal_reader.tal_struct_array['tagName'])
        info = mne.create_info(ch_names, self.samplerate, ch_types='eeg')

        #Reorganize data for MNE format
        data = bp_eegs_filtered.transpose('events', 'bipolar_pairs', 'time')

        #Create events array for MNE
        mne_evs = np.empty([data['events'].shape[0], 3]).astype(int)
        mne_evs[:, 0] = np.arange(data['events'].shape[0])
        mne_evs[:, 1] = data['time'].shape[0]
        if 'PAL' in self.task:
            mne_evs[:, 2] = np.array(bp_eegs['events'])['correct']
        else:
            mne_evs[:, 2] = np.array(bp_eegs['events'])['recalled']

        event_id = dict(recalled=1, not_recalled=0)
        tmin=0.0

        arr = mne.EpochsArray(np.array(data), info, mne_evs, tmin, event_id)

        #Assess connectivity
        fmin, fmax = freq[0], freq[1]
        sfreq = arr.info['sfreq']  # the sampling frequency
        tmin = 0.0
        cons_rec = []
        for idx in range(len(arr['recalled'])):
            if self.mode=='cwt_morlet':
                con, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(
                    arr['recalled'][idx], method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                    faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False, cwt_frequencies=cwtfreq)
            else:
                con, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(
                    arr['recalled'][idx], method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                    faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False,)
                
            con = con[:, :, 0] #no need for last dimension
            cons_rec.append(con)
            print idx

        cons_rec = np.array(cons_rec)

        cons_nrec = []
        for idx in range(len(arr['not_recalled'])):
            if self.mode=='cwt_morlet':
                con, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(
                arr['not_recalled'][idx], method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False,  cwt_frequencies=cwtfreq)
            else:
                con, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(
                arr['not_recalled'][idx], method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False,)
            
            con = con[:, :, 0] #no need for last dimension
            cons_nrec.append(con)
            print idx

        cons_nrec = np.array(cons_nrec)

        #Save recalled, not recalled

        import os
        if self.montage=='0':
            fn = self.root+'/'+sub+'/'+sub+'_mne_'+self.conn_type+'_'+self.mode+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        else:
            fn = self.root+'/'+sub+'_'+str(self.montage)+'/'+sub+'_'+str(self.montage)+'_mne_'+self.conn_type+'_'+self.mode+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        self.dat_file = fn
        try:
            np.save(fn, [cons_rec, cons_nrec])
        except:
            if self.montage=='0':
                os.mkdir(self.root+'/'+sub+'/')
            else:
                os.mkdir(self.root+'/'+sub+'_'+str(self.montage)+'/')
            np.save(fn, [cons_rec, cons_nrec])
            
    
    def mne_coh_pipeline_surr(self,):
        #Add Echobase
        import sys
        sys.path.insert(0, '/home1/esolo/Echobase/')
        from Echobase.Sigproc.ts_surr import AAFTsur
        
        if self.mode=='cwt_morlet':
            if self.band=='theta':
                freq=[3., 8.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 1)
            elif self.band=='beta':
                freq=[15., 25.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='lowgamma':
                freq=[30., 40.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='highgamma':
                freq=[90., 105.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            else:
                raise ValueError
        else:
            if self.band=='theta':
                freq=[3., 8.]
            elif self.band=='beta':
                freq=[15., 25.]
            elif self.band=='lowgamma':
                freq=[30., 40.]
            elif self.band=='highgamma':
                freq=[90., 105.]
            else:
                raise ValueError

        sub = self.s
        self.freq = freq

        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate
        params['freq'] = freq

        #Get tal struct
        tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
        tal_reader = TalReader(filename=tal_path)
        monopolar_channels = tal_reader.get_monopolar_channels()
        bipolar_pairs = tal_reader.get_bipolar_pairs()

        #Get events
        e_path = '/data/events/'+self.task+'/'+sub+'_events.mat'
        base_e_reader = BaseEventReader(filename=e_path, eliminate_events_with_no_eeg=True)
        base_events = base_e_reader.read()
        evs = base_events[base_events.type=='WORD']

        #Get EEGs
        eeg_reader = EEGReader(events=evs, channels=monopolar_channels,
                              start_time=self.win_st, end_time=self.win_fin, buffer_time=0.0)
        base_eegs = eeg_reader.read()

        #Get bipolar timeseries
        m2b = MonopolarToBipolarMapper(time_series=base_eegs, bipolar_pairs=bipolar_pairs)
        bp_eegs = m2b.filter()

        if sub[0:2] == 'FR':
            freq_range = [48., 52.]
        else:
            freq_range = [58., 62.]
        b_filter = ButterworthFilter(time_series=bp_eegs, freq_range=freq_range, filt_type='stop', order=4)
        bp_eegs_filtered = b_filter.filter()

        #Downsample for speed
        bp_eegs_filtered = bp_eegs_filtered.resampled(self.samplerate)

        #Create MNE dataset
        n_channels = bp_eegs_filtered.shape[0]
        ch_names = list(tal_reader.tal_struct_array['tagName'])
        info = mne.create_info(ch_names, self.samplerate, ch_types='eeg')

        #Reorganize data for MNE format
        data = bp_eegs_filtered.transpose('events', 'bipolar_pairs', 'time')
        self.data = data

        #Create events array for MNE
        mne_evs = np.empty([data['events'].shape[0], 3]).astype(int)
        mne_evs[:, 0] = np.arange(data['events'].shape[0])
        mne_evs[:, 1] = data['time'].shape[0]
        mne_evs[:, 2] = evs.recalled

        event_id = dict(recalled=1, not_recalled=0)
        tmin=0.0

        arr = mne.EpochsArray(np.array(data), info, mne_evs, tmin, event_id)

        #Assess connectivity
        fmin, fmax = freq[0], freq[1]
        sfreq = arr.info['sfreq']  # the sampling frequency
        tmin = 0.0

        cons, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(
                    arr, method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                    faverage=True, tmin=tmin, mt_adaptive=False, n_jobs=1, verbose=False,)
        cons = cons[:, :, 0] #don't need last dim if averaging freqs
        
        #Save data
        import os
        fn = self.root+'/'+sub+'/'+sub+'_mne_alltrials_'+self.conn_type+'_'+self.mode+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        fn_data = self.root+'/'+sub+'/'+sub+'_mne_alltrials_data_'+self.conn_type+'_'+self.mode+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        self.dat_file = fn
        self.stats = cons

        try:
            np.save(fn, cons)
            np.save(fn_data, data)
        except:
            os.mkdir('/scratch/esolo/mne_analysis/'+sub+'/')
            np.save(fn, cons)
            np.save(fn_data, data)
            
        return
            

    def pool_elecs(self, shuffle_func, nrand=500, pyFRsubs=pyFRsubs):

        import cPickle as pk
        import numpy as np
        from scipy.io import loadmat
        from glob import glob
        import itertools
        from ptsa.data.readers import BaseEventReader
        from ptsa.data.readers.TalReader import TalReader
        from ptsa.data.common import xr    

        def make_label(string):
            "Reformats string from location data in tal structure."
            import re
            result = re.findall('[a-zA-Z]+', string)
            label = ''
            for r in result:
                if r != 'u':
                    label += ' '+r
            return(label[1:])
            
        sub = self.s
        
        if shuffle_func.__name__!='return_preshuffled':
            try:
                M = np.load(self.root+'/'+sub+'/'+sub+'_mne_'+self.conn_type+'_'+self.mode+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy')
                
                rem = M[0]
                nrem = M[1]
            except:
                return 
        else:
            rem = []
            nrem = []

        #Get electrode data
        tal_path = str('/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat')
        tal_reader = TalReader(filename=tal_path)
        monopolar_channels = tal_reader.get_monopolar_channels()
        bipolar_pairs = tal_reader.get_bipolar_pairs()
        sub_elecs = tal_reader.tal_struct_array      

        #Get hippocampal elecs
        if sub in pyFRsubs:
            all_elecs = np.load('/scratch/esolo/all_good_elecs.npy')
            pyFR_e = all_elecs[all_elecs['subject']==sub]
            for e in pyFR_e:
                elecnum = e['elecnum'][0][0]-1
                if e['Loc3']=='Hippocampus':
                    print 'Hipp elec found!'
                    sub_elecs[elecnum]['Loc3']='Hippocampus'

        if sub not in pyFRsubs:
            try:
                Hipp_locTag = ['Left CA1','Left CA2','Left CA3','Left DG','Left Sub',
                'Right CA1','Right CA2','Right CA3','Right DG','Right Sub']
                for i in range(0, len(sub_elecs)):
                    if sub_elecs[i].locTag in Hipp_locTag:
                        sub_elecs[i]['Loc3']='Hippocampus'
            except:
                pass

        label_ord = pk.load(open('/scratch/esolo/labels_ordered.pk', 'rb')) #labeling ordered by lobe
        adj = np.zeros([nrand+1, len(label_ord), len(label_ord)])
        adj_sub = np.zeros([nrand+1, len(sub_elecs), len(sub_elecs)])
        adjC = np.zeros([len(label_ord), len(label_ord)])
        combs = list(itertools.combinations(range(0, len(sub_elecs)), 2))

        #Figure out which electrodes to exclude due to shared monopolar contacts
        pairs_to_skip = []
        for c in combs:
            t = list(sub_elecs[c[0]]['channel'])
            t.extend(sub_elecs[c[1]]['channel'])
            if len(np.unique(t))<4:
                pairs_to_skip.append(c)

        Md = shuffle_func(rem, nrem, nrand)

        for c in combs:

            if c in pairs_to_skip:
                continue

            elec1 = c[0]
            elec2 = c[1]

            #Make labels for lookup table
            region1_str = str(sub_elecs[elec1]['Loc1'])+' '+str(sub_elecs[elec1]['Loc3'])
            region2_str = str(sub_elecs[elec2]['Loc1'])+' '+str(sub_elecs[elec2]['Loc3'])

            region1_label = make_label(region1_str)
            region2_label = make_label(region2_str)

            #Ignore comparison if doesn't show up in usable list
            try:
                region1_idx = label_ord.index(region1_label)
                region2_idx = label_ord.index(region2_label)
            except ValueError:
                continue

            #Need to sort region indices to use only upper-triangle of matrix
            r_list = [region1_idx, region2_idx]
            r_list.sort()

            adj[:, r_list[0], r_list[1]] += Md[:, elec2, elec1]
            adj_sub[:, elec1, elec2] += Md[:, elec2, elec1]
            adjC[r_list[0], r_list[1]] += 1

            self.stats_file = self.root+'/'+sub+'/'+sub+'_mne_'+self.conn_type+'_'+self.mode+'_'+shuffle_func.__name__+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
            
        np.save(self.stats_file, [adj, adjC, adj_sub])
        return
    
    def mne_PLI_pipeline(self, nRandomizations=500, match_trial_n=False, threshold_n=False):

        if self.mode=='cwt_morlet':
            if self.band=='theta':
                freq=[3., 8.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 1)
            elif self.band=='beta':
                freq=[15., 25.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='lowgamma':
                freq=[32., 48.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='highgamma':
                freq=[90., 105.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='all_freqs':
                freq = [3., 130.]
                cwtfreq = np.array([3, 4, 5, 6, 7, 8, 
                                    9, 10, 11, 12, 13, 
                                    16, 18, 20, 22, 24, 26, 28, 
                                    30, 35, 40, 45, 50, 55, 60, 
                                    90, 100, 110, 120, 130])
            else:
                raise ValueError
        else:
            if self.band=='theta':
                freq=[3., 8.]
            elif self.band=='beta':
                freq=[15., 25.]
            elif self.band=='lowgamma':
                freq=[32., 48.]
            elif self.band=='highgamma':
                freq=[90., 105.]
            else:
                raise ValueError

        sub = self.s

        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate
        params['freq'] = freq

        #Get tal struct
        tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
        tal_reader = TalReader(filename=tal_path)
        monopolar_channels = tal_reader.get_monopolar_channels()
        bipolar_pairs = tal_reader.get_bipolar_pairs()

        #Get events
        e_path = '/data/events/'+self.task+'/'+sub+'_events.mat'
        base_e_reader = BaseEventReader(filename=e_path, eliminate_events_with_no_eeg=True)
        base_events = base_e_reader.read()
        evs = base_events[base_events.type=='WORD']
        self.evs = evs

        #Get EEGs
        eeg_reader = EEGReader(events=evs, channels=monopolar_channels,
                              start_time=self.win_st, end_time=self.win_fin, buffer_time=1.0)
        base_eegs = eeg_reader.read()

        #Get bipolar timeseries
        m2b = MonopolarToBipolarMapper(time_series=base_eegs, bipolar_pairs=bipolar_pairs)
        bp_eegs = m2b.filter()

        if sub[0:2] == 'FR':
            freq_range = [48., 52.]
        else:
            freq_range = [58., 62.]
        b_filter = ButterworthFilter(time_series=bp_eegs, freq_range=freq_range, filt_type='stop', order=4)
        bp_eegs_filtered = b_filter.filter()

        #Downsample for speed
        if int(bp_eegs.samplerate) > 500:
            bp_eegs_filtered = bp_eegs_filtered.resampled(self.samplerate)
        elif int(bp_eegs.samplerate) < freq[1]*2.0:
            raise('Sampling rate too low!!')
            return
        else:
            self.samplerate = int(bp_eegs.samplerate)
            
        #Create MNE dataset
        n_channels = bp_eegs_filtered.shape[0]
        ch_names = list(tal_reader.tal_struct_array['tagName'])
        info = mne.create_info(ch_names, self.samplerate, ch_types='eeg')

        #Reorganize data for MNE format
        data = bp_eegs_filtered.transpose('events', 'bipolar_pairs', 'time')

        #Create events array for MNE
        mne_evs = np.empty([data['events'].shape[0], 3]).astype(int)
        mne_evs[:, 0] = np.arange(data['events'].shape[0])
        mne_evs[:, 1] = data['time'].shape[0]
        mne_evs[:, 2] = list(evs.recalled)

        event_id = dict(recalled=1, not_recalled=0)
        tmin=0.0

        arr = mne.EpochsArray(np.array(data), info, mne_evs, tmin, event_id)
        self.mne_arr = arr

        #Assess connectivity
        fmin, fmax = freq[0], freq[1]
        sfreq = arr.info['sfreq']  # the sampling frequency
        tmin = 0.0
        
        rec_idxs = np.arange(len(arr['recalled']))
        nrec_idxs = np.arange(len(arr['not_recalled']))
        if match_trial_n is True:
            
            #If more remembered than not rememebred, shrink to not-remembered size with random shuffle
            if rec_idxs.size > nrec_idxs.size:
                np.random.shuffle(rec_idxs)
                rec_idxs = rec_idxs[:nrec_idxs.size]
                
            if nrec_idxs.size > rec_idxs.size:
                np.random.shuffle(nrec_idxs)
                nrec_idxs = nrec_idxs[:rec_idxs.size]
                
        if threshold_n is True:
            if len(rec_idxs) < 75 or len(nrec_idxs) < 75:
                raise ValueError('Insufficient Trial Count!')
                return
            
        print 'Computing for '+str(rec_idxs.size)+' remembered and '+str(nrec_idxs.size)+' not-rememebred events.'
                                 
        if self.mode=='cwt_morlet':
            cons_rec, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(arr['recalled'][rec_idxs],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=False, n_jobs=1, verbose=True, cwt_frequencies=cwtfreq, 
                                                        block_size=2000, cwt_n_cycles=6.0)
 
            cons_nrec, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(
                arr['not_recalled'][nrec_idxs], method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=False, n_jobs=1, verbose=True, cwt_frequencies=cwtfreq, 
                                                        block_size=2000, cwt_n_cycles=6.0)
        else:
            cons_rec, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(arr['recalled'][rec_idxs],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False, 
                                                        block_size=2000)
            cons_rec = cons_rec[:, :, 0]

            cons_nrec, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(
                arr['not_recalled'][nrec_idxs], method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False, 
                                                        block_size=2000)
            cons_nrec = cons_nrec[:, :, 0]            

        self.cons_rec = cons_rec
        self.cons_nrec = cons_nrec
        
        fn = self.root+''+sub+'/'+sub+'_mne_'+str(self.conn_type)+'_'+self.mode+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        
        import os
        try:
            os.mkdir('/scratch/esolo/mne_analysis/'+sub+'/')
            np.save(fn, [cons_rec, cons_nrec])
        except:
            np.save(fn, [cons_rec, cons_nrec])
            
        if nRandomizations==0:
            return

        n1 = float(np.sum(evs.recalled==1)); n2 = float(np.sum(evs.recalled==0))
        #f_ = ((n1-1)*(n2-(n2*cons_nrec)))/((n2-1)*(n1-(n1*cons_rec)))
        f_ = cons_rec-cons_nrec
        
        fstats = []
        fstats.append(f_)
        
        #Load preshuffled vectors
        try:
            shuffled_vecs = np.load('/scratch/esolo/mne_analysis/'+sub+'/shuffled_vecs.npy')
        except:
            from copy import copy
            shuffled_vecs = []
            all_idxs = np.arange(len(evs))
                
            for k in range(nRandomizations):
                np.random.shuffle(all_idxs)
                shuffled_vecs.append(copy(all_idxs))
                
            shuffled_vecs = np.array(shuffled_vecs)
            np.save('/scratch/esolo/mne_analysis/'+sub+'/shuffled_vecs.npy', shuffled_vecs)
            
        for k in range(nRandomizations):
            
            idxs = shuffled_vecs[k, :]
            
            if self.mode=='cwt_morlet':
                
                null_rec, _, _, _, _ = mne.connectivity.spectral_connectivity(arr[idxs[:int(n1)]],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=False, n_jobs=1, verbose=True, cwt_frequencies=cwtfreq, cwt_n_cycles=6.0,
                                        block_size=2000)

                null_nrec, _, _, _, _ = mne.connectivity.spectral_connectivity(arr[idxs[int(n1):]],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=False, n_jobs=1, verbose=True, cwt_frequencies=cwtfreq, cwt_n_cycles=6.0,
                                        block_size=2000)
            
            else:
                
                null_rec, _, _, _, _ = mne.connectivity.spectral_connectivity(arr[idxs[:int(n1)]],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False, 
                                        block_size=2000)

                null_nrec, _, _, _, _ = mne.connectivity.spectral_connectivity(arr[idxs[int(n1):]],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False, 
                                        block_size=2000)

            #f_null = ((n1-1)*(n2-(n2*null_nrec[:, :, 0])))/((n2-1)*(n1-(n1*null_rec[:, :, 0])))
            f_null = null_rec[:, :, 0]-null_nrec[:, :, 0]
            fstats.append(f_null)

            print k
            
        fstats = np.array(fstats)    
        self.fstats = fstats

        #wpli2_debiased

        #Save recalled, not recalled

        import os
        fn = self.root+''+sub+'/'+sub+'_mne_'+str(self.conn_type)+'_'+self.mode+'_fstat_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        self.stats_file = fn

        try:
            np.save(fn, fstats)
        except:
            try:
                os.mkdir('/scratch/esolo/mne_analysis/'+sub+'/')
                np.save(fn, fstats)
            except:
                np.save(fn, fstats)
                
    def mne_PLI_pipeline_MTL(self, nRandomizations=500, match_trial_n=False, threshold_n=False):
        from ptsa.data.events import Events
        from ptsa.data.readers.IndexReader import JsonIndexReader
        reader = JsonIndexReader('/protocols/r1.json')
        from skimage.util import view_as_blocks
        
        MTL_labels = ['Left CA1',
         'Left CA2',
         'Left CA3',
         'Left DG',
         'Left Sub',
         'Left PRC',
         'Left EC',
         'Left PHC',
         'Right CA1',
         'Right CA2',
         'Right CA3',
         'Right DG',
         'Right Sub',
         'Right PRC',
         'Right EC',
         'Right PHC']

        if self.mode=='cwt_morlet':
            if self.band=='lowtheta':
                freq=[3., 4.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 1)
            elif self.band=='theta':
                freq=[3., 8.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 1)
            elif self.band=='beta':
                freq=[15., 25.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='lowgamma':
                freq=[20., 60.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='highgamma':
                freq=[90., 105.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='all_freqs':
                freq = [3., 128.]
                #cwtfreq = (2.**(1./8.))**np.arange(13., 57.)
                cwtfreq = np.array([3., 4., 5., 6., 7., 8.,
                                    9., 10., 11., 12., 13.,
                                    16., 18., 20., 22., 24., 26., 28., 
                                    30., 32., 34., 36., 38., 40., 42., 44., 46., 48.,
                                    50., 55., 60., 65., 70., 75., 80.])
            else:
                raise ValueError
        else:
            if self.band=='theta':
                freq=[3., 8.]
            elif self.band=='beta':
                freq=[15., 25.]
            elif self.band=='lowgamma':
                freq=[32., 48.]
            elif self.band=='highgamma':
                freq=[90., 105.]
            else:
                raise ValueError

        sub = self.s

        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate
        params['freq'] = freq

        #Get tal struct
        tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
        tal_reader = TalReader(filename=tal_path)
        monopolar_channels = tal_reader.get_monopolar_channels()
        bipolar_pairs = tal_reader.get_bipolar_pairs()

        #Get available montages
        montages = reader.montages(subject=sub, experiment=self.task)

        #Just use the first montage for the task to avoid confusion
        montage=montages[0]
        sessions = reader.sessions(subject=sub, experiment=self.task, montage=montage)

        #Get events
        evfiles = list(reader.aggregate_values('task_events', subject=sub, experiment=self.task, montage=montage))

        evs = np.array([])
        for ef in evfiles:
            base_e_reader = BaseEventReader(filename=ef, eliminate_events_with_no_eeg=True)
            base_events = base_e_reader.read()
            if len(evs) == 0:
                evs = base_events[base_events.type=='WORD']
            else:
                evs = np.concatenate((evs, base_events[base_events.type=='WORD']), axis=0)
        evs = Events(evs)
        self.evs = evs
        
        #Get EEGs
        eeg_reader = EEGReader(events=evs, channels=monopolar_channels,
                              start_time=self.win_st, end_time=self.win_fin, buffer_time=1.0)
        base_eegs = eeg_reader.read()

        #Get bipolar timeseries
        m2b = MonopolarToBipolarMapper(time_series=base_eegs, bipolar_pairs=bipolar_pairs)
        bp_eegs = m2b.filter()
        
        #Only use MTL electrodes
        MTLelecs = [elecidx for elecidx, l in enumerate(tal_reader.tal_struct_array.locTag) if l in MTL_labels]
        
        if len(MTLelecs) < 2:
            print 'Less than 2 MTL electrodes!'
            return
        
        bp_eegs = bp_eegs[MTLelecs, :, :]
        MTL_info = tal_reader.tal_struct_array[MTLelecs]
        np.save(self.root+''+sub+'/MTL_info.npy', MTL_info)

        if sub[0:2] == 'FR':
            freq_range = [48., 52.]
        else:
            freq_range = [58., 62.]
        b_filter = ButterworthFilter(time_series=bp_eegs, freq_range=freq_range, filt_type='stop', order=4)
        bp_eegs_filtered = b_filter.filter()
        
        #Downsample for speed
#         if int(bp_eegs.samplerate) > 500:
#             bp_eegs_filtered = bp_eegs_filtered.resampled(self.samplerate)
#         elif int(bp_eegs.samplerate) < freq[1]*2.0:
#             raise('Sampling rate too low!!')
#             return
#         else:
#             self.samplerate = int(bp_eegs.samplerate)

        bp_eegs_filtered = bp_eegs_filtered.resampled(self.samplerate)
            
        #Create MNE dataset
        n_channels = bp_eegs_filtered.shape[0]
        ch_names = list(MTL_info['tagName'])
        info = mne.create_info(ch_names, self.samplerate, ch_types='eeg')

        #Reorganize data for MNE format
        data = bp_eegs_filtered.transpose('events', 'bipolar_pairs', 'time')

        #Create events array for MNE
        mne_evs = np.empty([data['events'].shape[0], 3]).astype(int)
        mne_evs[:, 0] = np.arange(data['events'].shape[0])
        mne_evs[:, 1] = data['time'].shape[0]
        mne_evs[:, 2] = list(evs.recalled)

        event_id = dict(recalled=1, not_recalled=0)
        tmin=0.0

        arr = mne.EpochsArray(np.array(data), info, mne_evs, tmin, event_id)
        self.mne_arr = arr
        
        #Assess connectivity
        fmin, fmax = freq[0], freq[1]
        sfreq = arr.info['sfreq']  # the sampling frequency
        tmin = 0.0
        
        rec_idxs = np.arange(len(arr['recalled']))
        nrec_idxs = np.arange(len(arr['not_recalled']))
        if match_trial_n is True:
            
            #If more remembered than not rememebred, shrink to not-remembered size with random shuffle
            if rec_idxs.size > nrec_idxs.size:
                np.random.shuffle(rec_idxs)
                rec_idxs = rec_idxs[:nrec_idxs.size]
                
            if nrec_idxs.size > rec_idxs.size:
                np.random.shuffle(nrec_idxs)
                nrec_idxs = nrec_idxs[:rec_idxs.size]
                
        if threshold_n is True:
            if len(rec_idxs) < 30 or len(nrec_idxs) < 30:
                raise ValueError('Insufficient Trial Count!')
                return
            
        self.arr = arr
            
        print 'Computing for '+str(rec_idxs.size)+' remembered and '+str(nrec_idxs.size)+' not-rememebred events.'
                                 
        if self.mode=='cwt_morlet':
            cons_rec, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(arr['recalled'][rec_idxs],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=False, n_jobs=1, verbose=True, cwt_frequencies=cwtfreq, cwt_n_cycles=6.0)
 
            cons_nrec, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(
                arr['not_recalled'][nrec_idxs], method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=False, n_jobs=1, verbose=True, cwt_frequencies=cwtfreq, cwt_n_cycles=6.0)
        else:
            cons_rec, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(arr['recalled'][rec_idxs],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False, 
                                                        block_size=2000)
            cons_rec = cons_rec[:, :, 0]

            cons_nrec, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(
                arr['not_recalled'][nrec_idxs], method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False, 
                                                        block_size=2000)
            cons_nrec = cons_nrec[:, :, 0]            
        
        if nRandomizations==0:
            self.cons_rec = cons_rec
            self.cons_nrec = cons_nrec

            fn = self.root+''+sub+'/'+sub+'_mne_MTL_'+str(self.conn_type)+'_'+self.mode+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'

            import os
            try:
                os.mkdir('/scratch/esolo/mne_analysis/'+sub+'/')
                np.save(fn, [cons_rec, cons_nrec])
            except:
                np.save(fn, [cons_rec, cons_nrec])
        else:
            pass
            
        if nRandomizations==0:
            return

        fstats = []
        n1 = float(np.sum(evs.recalled==1)); n2 = float(np.sum(evs.recalled==0))
        #f_ = ((n1-1)*(n2-(n2*cons_nrec)))/((n2-1)*(n1-(n1*cons_rec)))
        
        #Sample rate and time check
        #if cons_rec[0].shape[3]!=(2.0+self.win_fin-self.win_st)*self.samplerate:
        #        print 'Time samples dont match expected'
        #        return
        
        if type(self.conn_type)==str:
            
            binsize=int(self.samplerate/10.0)
            total_wins = cons_rec.shape[3]/binsize
            diff = cons_rec-cons_nrec
            diff_avg = np.reshape(diff[:, :, :, :binsize*total_wins],
                   (cons_rec.shape[0], cons_rec.shape[1], cons_rec.shape[2], total_wins, binsize))
            diff_avg = np.mean(diff_avg, axis=4)
            
            fstats.append(diff_avg)
        else:
            for i in range(len(cons_rec)):
                fstats.append([])            
                binsize=int(self.samplerate/10.0)
                total_wins = cons_rec[i].shape[3]/binsize
                diff = cons_rec[i]-cons_nrec[i]
                diff_avg = np.reshape(diff[:, :, :, :binsize*total_wins],
                   (diff.shape[0], diff.shape[1], diff.shape[2], total_wins, binsize))
                diff_avg = np.mean(diff_avg, axis=4)
                fstats[i].append(diff_avg)

        #fstats.append(np.array(cons_rec)-np.array(cons_nrec))
        
        #Load preshuffled vectors
        shuffled_vecs = np.load('/scratch/esolo/mne_analysis/'+sub+'/shuffled_vecs.npy')
        
        for k in range(nRandomizations):
            
            if match_trial_n:
                idxs = np.arange(len(rec_idxs)+len(nrec_idxs))
                np.random.shuffle(idxs)
                n1 = len(rec_idxs)
            else:
                idxs = shuffled_vecs[k, :]
            
            if self.mode=='cwt_morlet':
                
                null_rec, _, _, _, _ = mne.connectivity.spectral_connectivity(arr[idxs[:int(n1)]],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=False, n_jobs=1, verbose=False, cwt_frequencies=cwtfreq, 
                                        block_size=2000, cwt_n_cycles=6.0)

                null_nrec, _, _, _, _ = mne.connectivity.spectral_connectivity(arr[idxs[int(n1):]],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=False, n_jobs=1, verbose=False, cwt_frequencies=cwtfreq, 
                                        block_size=2000, cwt_n_cycles=6.0)
            
            else:
                
                null_rec, _, _, _, _ = mne.connectivity.spectral_connectivity(arr[idxs[:int(n1)]],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False, 
                                        block_size=2000)

                null_nrec, _, _, _, _ = mne.connectivity.spectral_connectivity(arr[idxs[int(n1):]],
                method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
                faverage=True, tmin=tmin, mt_adaptive=True, n_jobs=1, verbose=False, 
                                        block_size=2000)

            #f_null = ((n1-1)*(n2-(n2*null_nrec[:, :, 0])))/((n2-1)*(n1-(n1*null_rec[:, :, 0])))
            #f_null = null_rec[:, :, 0]-null_nrec[:, :, 0]
            
            #Get average of the difference in 200ms windows
            if type(self.conn_type)==str:
                null_diff = null_rec-null_nrec
                null_diff_avg = np.reshape(null_diff[:, :, :, :binsize*total_wins],
                                          (null_diff.shape[0], null_diff.shape[1], null_diff.shape[2], total_wins, binsize))
                
                null_diff_avg = np.mean(null_diff_avg, axis=4)
                fstats.append(null_diff_avg)
            else:
                for i in range(len(null_rec)):

                    null_diff = null_rec[i]-null_nrec[i]
                    null_diff_avg = np.reshape(null_diff[:, :, :, :binsize*total_wins],
                          (null_diff.shape[0], null_diff.shape[1], null_diff.shape[2], total_wins, binsize))
                    null_diff_avg = np.mean(null_diff_avg, axis=4)

                    fstats[i].append(null_diff_avg)
            print k
        
            #fstats.append(np.array(null_rec)-np.array(null_nrec))
        
        fstats = np.array(fstats)    
        self.fstats = fstats

        import os
        fn = self.root+''+sub+'/'+sub+'_mne_MTL_100ms_'+str(self.conn_type)+'_'+self.mode+'_dstat_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        self.stats_file = fn

        try:
            np.save(fn, fstats)
        except:
            try:
                os.mkdir('/scratch/esolo/mne_analysis/'+sub+'/')
                np.save(fn, fstats)
            except:
                np.save(fn, fstats)

    def mne_pipeline_MTL_powers_avgref(self, match_trial_n=False, threshold_n=False):
        from ptsa.data.events import Events
        from ptsa.data.readers.IndexReader import JsonIndexReader
        reader = JsonIndexReader('/protocols/r1.json')
        
        MTL_labels = ['Left CA1', 'Left CA2', 'Left CA3', 'Left DG', 'Left Sub', 'Left PRC', 'Left EC', 'Left PHC',
         'Right CA1', 'Right CA2', 'Right CA3', 'Right DG', 'Right Sub', 'Right PRC', 'Right EC', 'Right PHC']

        #must be done with mode=cwt_morlet
        if self.band=='lowtheta':
            freq=[3., 4.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='theta':
            freq=[3., 8.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='beta':
            freq=[15., 25.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='lowgamma':
            freq=[20., 60.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='highgamma':
            freq=[90., 105.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='all_freqs':
            freq = [3., 128.]
            #cwtfreq = (2.**(1./8.))**np.arange(13., 57.)
            cwtfreq = np.array([3., 4., 5., 6., 7., 8.,
                                9., 10., 11., 12., 13.,
                                16., 18., 20., 22., 24., 26., 28., 
                                30., 32., 34., 36., 38., 40., 42., 44., 46., 48.,
                                50., 55., 60., 65., 70., 75., 80.])
        elif self.band=='thetaalphabeta':
            freq = [3., 128.]
            #cwtfreq = (2.**(1./8.))**np.arange(13., 57.)
            cwtfreq = np.array([3., 4., 5., 6., 7., 8.,
                                9., 10., 11., 12., 13.,
                                16., 18., 20., 22., 24., 26., 28.,])
        elif self.band=='lowgammahighgamma':
            freq = [3., 128.]
            #cwtfreq = (2.**(1./8.))**np.arange(13., 57.)
            cwtfreq = np.array([30., 32., 34., 36., 38., 40., 42., 44., 46., 48.,
                                50., 55., 60., 65., 70., 75., 80.])
        else:
            return

        sub = self.s
        
        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate
        params['freq'] = freq
        
        #Get tal struct
        from scipy.io import loadmat
        tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_monopol.mat'
        tal_struct = TalReader(filename=tal_path, struct_name='talStruct').read()
        tal_path_bipol = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
        tal_reader = TalReader(filename=tal_path_bipol)
        monopolar_channels = tal_reader.get_monopolar_channels()
        bipolar_pairs = tal_reader.get_bipolar_pairs()

        bpinc, mpinc = exclude_bad([tal_path], [bipolar_pairs], [monopolar_channels])
        good_chan_idxs = [list(monopolar_channels).index(c) for c in mpinc[0]]
        tal_struct = tal_struct[good_chan_idxs]
        
        #Only use MTL electrodes
        MTLelecs = [elecidx for elecidx, l in enumerate(tal_struct['locTag']) if l in MTL_labels]
        
        if len(MTLelecs) < 2:
            print 'Less than 2 MTL electrodes!'
            return

        #Get available montages
        montages = reader.montages(subject=sub, experiment=self.task)

        #Just use the first montage for the task to avoid confusion
        montage=montages[0]
        sessions = reader.sessions(subject=sub, experiment=self.task, montage=montage)

        #Get events
        evfiles = list(reader.aggregate_values('task_events', subject=sub, experiment=self.task, montage=montage))

        evs = np.array([])
        for ef in evfiles:
            base_e_reader = BaseEventReader(filename=ef, eliminate_events_with_no_eeg=True)
            base_events = base_e_reader.read()
            if len(evs) == 0:
                evs = base_events[base_events.type=='WORD']
            else:
                evs = np.concatenate((evs, base_events[base_events.type=='WORD']), axis=0)
        evs = Events(evs)

        #Get EEGs
        eeg_reader = EEGReader(events=evs, channels=monopolar_channels[good_chan_idxs],
                              start_time=self.win_st, end_time=self.win_fin, buffer_time=1.0)
        base_eegs = eeg_reader.read()
        
        MTL_info = tal_struct[MTLelecs]
        np.save(self.root+''+sub+'/MTL_info_AvgRef.npy', MTL_info)

        eegs = base_eegs[:, :, :]

        if sub[0:2] == 'FR':
            freq_range = [48., 52.]
        else:
            freq_range = [58., 62.]
        b_filter = ButterworthFilter(time_series=eegs, freq_range=freq_range, filt_type='stop', order=4)
        eegs_filtered = b_filter.filter()
        eegs_filtered = eegs_filtered.resampled(self.samplerate)

        #Create MNE dataset
        n_channels = eegs_filtered.shape[0]
        ch_names = list(tal_struct['tagName'])
        info = mne.create_info(ch_names, self.samplerate, ch_types='eeg')

        #Reorganize data for MNE format
        data = eegs_filtered.transpose('events', 'channels', 'time')

        #Create events array for MNE
        mne_evs = np.empty([data['events'].shape[0], 3]).astype(int)
        mne_evs[:, 0] = np.arange(data['events'].shape[0])
        mne_evs[:, 1] = data['time'].shape[0]
        mne_evs[:, 2] = list(evs.recalled)

        event_id = dict(recalled=1, not_recalled=0)
        tmin=0.0

        arr = mne.EpochsArray(np.array(data), info, mne_evs, tmin, event_id)
        arr.set_eeg_reference(ref_channels=None) #Set to average reference
        arr.apply_proj()
        
        from mne.time_frequency import tfr_array_morlet
        sfreq = arr.info['sfreq']  # the sampling frequency
        powers = tfr_array_morlet(arr, sfreq=sfreq, frequencies=cwtfreq, n_cycles=6.0, output='power', verbose=True)
        powers_MTL = powers[:, MTLelecs, :, :]
        
        powers_MTL = np.log10(powers_MTL) #take log for low-power decay at high freqs

        #z-score by sessions 
        from scipy.stats import zscore
        for s in np.unique(evs.session):
            #z-score powers by session
            vals = powers_MTL[np.array(eegs['events'])['session']==s, :, :, :]
            valsz = zscore(vals, axis=0, ddof=1)
            powers_MTL[np.array(eegs['events'])['session']==s, :, :, :] = valsz
        
        np.save(self.root+'/'+sub+'/'+sub+'_powers_MNE_AvgRef_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy',
                powers_MTL)
        np.save(self.root+'/'+sub+'/Events_'+self.task+'.npy', evs.recalled)
                
                
    def mne_pipeline_MTL_phases_avgref(self, match_trial_n=False, threshold_n=False):
        from ptsa.data.events import Events
        from ptsa.data.readers.IndexReader import JsonIndexReader
        reader = JsonIndexReader('/protocols/r1.json')
        
        MTL_labels = ['Left CA1', 'Left CA2', 'Left CA3', 'Left DG', 'Left Sub', 'Left PRC', 'Left EC', 'Left PHC',
         'Right CA1', 'Right CA2', 'Right CA3', 'Right DG', 'Right Sub', 'Right PRC', 'Right EC', 'Right PHC']

        #must be done with mode=cwt_morlet
        if self.band=='lowtheta':
            freq=[3., 4.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='theta':
            freq=[3., 8.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='beta':
            freq=[15., 25.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='lowgamma':
            freq=[20., 60.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='highgamma':
            freq=[90., 105.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='all_freqs':
            freq = [3., 128.]
            #cwtfreq = (2.**(1./8.))**np.arange(13., 57.)
            cwtfreq = np.array([3., 4., 5., 6., 7., 8.,
                                9., 10., 11., 12., 13.,
                                16., 18., 20., 22., 24., 26., 28., 
                                30., 32., 34., 36., 38., 40., 42., 44., 46., 48.,
                                50., 55., 60., 65., 70., 75., 80.])
        elif self.band=='thetaalphabeta':
            freq = [3., 128.]
            #cwtfreq = (2.**(1./8.))**np.arange(13., 57.)
            cwtfreq = np.array([3., 4., 5., 6., 7., 8.,
                                9., 10., 11., 12., 13.,
                                16., 18., 20., 22., 24., 26., 28.,])
        elif self.band=='lowgammahighgamma':
            freq = [3., 128.]
            #cwtfreq = (2.**(1./8.))**np.arange(13., 57.)
            cwtfreq = np.array([30., 32., 34., 36., 38., 40., 42., 44., 46., 48.,
                                50., 55., 60., 65., 70., 75., 80.])
        else:
            return

        sub = self.s
        
        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate
        params['freq'] = freq
        
        #Get tal struct
        from scipy.io import loadmat
        tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_monopol.mat'
        tal_struct = TalReader(filename=tal_path, struct_name='talStruct').read()
        tal_path_bipol = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
        tal_reader = TalReader(filename=tal_path_bipol)
        monopolar_channels = tal_reader.get_monopolar_channels()
        bipolar_pairs = tal_reader.get_bipolar_pairs()

        bpinc, mpinc = exclude_bad([tal_path], [bipolar_pairs], [monopolar_channels])
        good_chan_idxs = [list(monopolar_channels).index(c) for c in mpinc[0]]
        tal_struct = tal_struct[good_chan_idxs]
        
        #Only use MTL electrodes
        MTLelecs = [elecidx for elecidx, l in enumerate(tal_struct['locTag']) if l in MTL_labels]
        
        if len(MTLelecs) < 2:
            print 'Less than 2 MTL electrodes!'
            return

        #Get available montages
        montages = reader.montages(subject=sub, experiment=self.task)

        #Just use the first montage for the task to avoid confusion
        montage=montages[0]
        sessions = reader.sessions(subject=sub, experiment=self.task, montage=montage)

        #Get events
        evfiles = list(reader.aggregate_values('task_events', subject=sub, experiment=self.task, montage=montage))

        evs = np.array([])
        for ef in evfiles:
            base_e_reader = BaseEventReader(filename=ef, eliminate_events_with_no_eeg=True)
            base_events = base_e_reader.read()
            if len(evs) == 0:
                evs = base_events[base_events.type=='WORD']
            else:
                evs = np.concatenate((evs, base_events[base_events.type=='WORD']), axis=0)
        evs = Events(evs)

        #Get EEGs
        eeg_reader = EEGReader(events=evs, channels=monopolar_channels[good_chan_idxs],
                              start_time=self.win_st, end_time=self.win_fin, buffer_time=1.0)
        base_eegs = eeg_reader.read()
        
        MTL_info = tal_struct[MTLelecs]
        np.save(self.root+''+sub+'/MTL_info_AvgRef.npy', MTL_info)

        eegs = base_eegs[:, :, :]

        if sub[0:2] == 'FR':
            freq_range = [48., 52.]
        else:
            freq_range = [58., 62.]
        b_filter = ButterworthFilter(time_series=eegs, freq_range=freq_range, filt_type='stop', order=4)
        eegs_filtered = b_filter.filter()
        eegs_filtered = eegs_filtered.resampled(self.samplerate)

        #Create MNE dataset
        n_channels = eegs_filtered.shape[0]
        ch_names = list(tal_struct['tagName'])
        info = mne.create_info(ch_names, self.samplerate, ch_types='eeg')

        #Reorganize data for MNE format
        data = eegs_filtered.transpose('events', 'channels', 'time')

        #Create events array for MNE
        mne_evs = np.empty([data['events'].shape[0], 3]).astype(int)
        mne_evs[:, 0] = np.arange(data['events'].shape[0])
        mne_evs[:, 1] = data['time'].shape[0]
        mne_evs[:, 2] = list(evs.recalled)

        event_id = dict(recalled=1, not_recalled=0)
        tmin=0.0

        arr = mne.EpochsArray(np.array(data), info, mne_evs, tmin, event_id)
        arr.set_eeg_reference(ref_channels=None) #Set to average reference
        arr.apply_proj()
        
        from mne.time_frequency import tfr_array_morlet
        sfreq = arr.info['sfreq']  # the sampling frequency
        phases = tfr_array_morlet(arr, sfreq=sfreq, frequencies=cwtfreq, n_cycles=6.0, output='phase', verbose=True)
        phases_MTL = phases[:, MTLelecs, :, :]
        
        np.save(self.root+'/'+sub+'/'+sub+'_phases_MNE_AvgRef_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy',
                phases_MTL)
        np.save(self.root+'/'+sub+'/Events_'+self.task+'.npy', evs.recalled)
                
    def mne_PLI_pipeline_MTL_avgref(self, nRandomizations=500, match_trial_n=False, threshold_n=False, avg_epochs=True):
        from ptsa.data.events import Events
        from ptsa.data.readers.IndexReader import JsonIndexReader
        reader = JsonIndexReader('/protocols/r1.json')
        
        MTL_labels = ['Left CA1', 'Left CA2', 'Left CA3', 'Left DG', 'Left Sub', 'Left PRC', 'Left EC', 'Left PHC',
         'Right CA1', 'Right CA2', 'Right CA3', 'Right DG', 'Right Sub', 'Right PRC', 'Right EC', 'Right PHC']

        #must be done with mode=cwt_morlet
        if self.band=='lowtheta':
            freq=[3., 4.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='theta':
            freq=[3., 8.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 1)
        elif self.band=='beta':
            freq=[15., 25.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='lowgamma':
            freq=[20., 60.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='highgamma':
            freq=[90., 105.]
            cwtfreq = np.arange(freq[0], freq[1]+1, 2)
        elif self.band=='all_freqs':
            freq = [3., 128.]
            #cwtfreq = (2.**(1./8.))**np.arange(13., 57.)
            cwtfreq = np.array([3., 4., 5., 6., 7., 8.,
                                9., 10., 11., 12., 13.,
                                16., 18., 20., 22., 24., 26., 28., 
                                30., 32., 34., 36., 38., 40., 42., 44., 46., 48.,
                                50., 55., 60., 65., 70., 75., 80.])

        sub = self.s
        
        #Set up some params
        params = {}
        params['window_start'] = self.win_st
        params['window_finish'] = self.win_fin
        params['samplerate'] = self.samplerate
        params['freq'] = freq
        
        #Get tal struct
        from scipy.io import loadmat
        tal_path = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_monopol.mat'
        tal_struct = loadmat(tal_path)['talStruct']
        tal_path_bipol = '/data/eeg/'+sub+'/tal/'+sub+'_talLocs_database_bipol.mat'
        tal_reader = TalReader(filename=tal_path_bipol)
        monopolar_channels = tal_reader.get_monopolar_channels()
        bipolar_pairs = tal_reader.get_bipolar_pairs()

        bpinc, mpinc = exclude_bad([tal_path], [bipolar_pairs], [monopolar_channels])
        good_chan_idxs = [list(monopolar_channels).index(c) for c in mpinc[0]]
        tal_struct = tal_struct[:, good_chan_idxs]
        
        #Only use MTL electrodes
        MTLelecs = [elecidx for elecidx, l in enumerate(tal_struct[0]['locTag']) if (len(l)>0) and (l[0] in MTL_labels)]

        if len(MTLelecs) < 2:
            print 'Less than 2 MTL electrodes!'
            raise ValueError

        #Get available montages
        montages = reader.montages(subject=sub, experiment=self.task)

        #Just use the first montage for the task to avoid confusion
        montage=montages[0]
        sessions = reader.sessions(subject=sub, experiment=self.task, montage=montage)

        #Get events
        evfiles = list(reader.aggregate_values('task_events', subject=sub, experiment=self.task, montage=montage))

        evs = np.array([])
        for ef in evfiles:
            base_e_reader = BaseEventReader(filename=ef, eliminate_events_with_no_eeg=True)
            base_events = base_e_reader.read()
            if len(evs) == 0:
                evs = base_events[base_events.type=='WORD']
            else:
                evs = np.concatenate((evs, base_events[base_events.type=='WORD']), axis=0)
        evs = Events(evs)

        #Get EEGs
        eeg_reader = EEGReader(events=evs, channels=monopolar_channels[good_chan_idxs],
                              start_time=self.win_st, end_time=self.win_fin, buffer_time=1.0)
        base_eegs = eeg_reader.read()

        eegs = base_eegs[:, :, :]
        MTL_info = tal_struct[0, MTLelecs]
        print MTL_info['locTag']

        if sub[0:2] == 'FR':
            freq_range = [48., 52.]
        else:
            freq_range = [58., 62.]
        b_filter = ButterworthFilter(time_series=eegs, freq_range=freq_range, filt_type='stop', order=4)
        eegs_filtered = b_filter.filter()
        eegs_filtered = eegs_filtered.resampled(self.samplerate)

        #Create MNE dataset
        n_channels = eegs_filtered.shape[0]
        ch_names = list([t[0] for t in tal_struct['tagName'][0]])
        info = mne.create_info(ch_names, self.samplerate, ch_types='eeg')

        #Reorganize data for MNE format
        data = eegs_filtered.transpose('events', 'channels', 'time')

        #Create events array for MNE
        mne_evs = np.empty([data['events'].shape[0], 3]).astype(int)
        mne_evs[:, 0] = np.arange(data['events'].shape[0])
        mne_evs[:, 1] = data['time'].shape[0]
        mne_evs[:, 2] = list(evs.recalled)

        event_id = dict(recalled=1, not_recalled=0)
        tmin=0.0

        arr = mne.EpochsArray(np.array(data), info, mne_evs, tmin, event_id)
        arr.set_eeg_reference(ref_channels=None) #Set to average reference
        arr.apply_proj()

        import itertools
        tmp = list(itertools.combinations(MTLelecs, 2))
        list1 = [i[0] for i in tmp]
        list2 = [i[1] for i in tmp]
        MTLinfo1 = [tal_struct[0, i[0]] for i in tmp]
        MTLinfo2 = [tal_struct[0, i[1]] for i in tmp]
        conn_inds = (list1, list2)
        import cPickle as pk
        np.save(self.root+''+sub+'/MTL_combs.npy', [np.array([MTLinfo1[idx]['locTag'][0] for idx in range(0, len(MTLinfo1))]), 
                np.array([MTLinfo2[idx]['locTag'][0] for idx in range(0, len(MTLinfo2))])])
        
        #Assess connectivity
        fmin, fmax = freq[0], freq[1]
        sfreq = arr.info['sfreq']  # the sampling frequency
        tmin = 0.0
        
        rec_idxs = np.arange(len(arr['recalled']))
        nrec_idxs = np.arange(len(arr['not_recalled']))
        if match_trial_n is True:
            
            #If more remembered than not rememebred, shrink to not-remembered size with random shuffle
            if rec_idxs.size > nrec_idxs.size:
                np.random.shuffle(rec_idxs)
                rec_idxs = rec_idxs[:nrec_idxs.size]
                
            if nrec_idxs.size > rec_idxs.size:
                np.random.shuffle(nrec_idxs)
                nrec_idxs = nrec_idxs[:rec_idxs.size]
                
        if threshold_n is True:
            if len(rec_idxs) < 30 or len(nrec_idxs) < 30:
                raise ValueError('Insufficient Trial Count!')
                return
            
        print 'Computing for '+str(rec_idxs.size)+' remembered and '+str(nrec_idxs.size)+' not-rememebred events.'
                                 
        cons_rec, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(arr['recalled'][rec_idxs],
            method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax, indices=conn_inds,
            faverage=False, n_jobs=1, verbose=True, cwt_frequencies=cwtfreq, cwt_n_cycles=6.0)

        cons_nrec, freqs, times, n_epochs, n_tapers = mne.connectivity.spectral_connectivity(
            arr['not_recalled'][nrec_idxs], method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax,
            indices=conn_inds, faverage=False, n_jobs=1, verbose=True, cwt_frequencies=cwtfreq, cwt_n_cycles=6.0)

        if nRandomizations==0:
            self.cons_rec = cons_rec
            self.cons_nrec = cons_nrec

            fn = self.root+''+sub+'/'+sub+'_mne_MTL_AvgRef_'+str(self.conn_type)+'_'+self.mode+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'

            import os
            try:
                os.mkdir(self.root+'/'+sub+'/')
                np.save(fn, [cons_rec, cons_nrec])
            except:
                np.save(fn, [cons_rec, cons_nrec])
        else:
            pass
            
        if nRandomizations==0:
            return

        fstats = []
        n1 = float(np.sum(evs.recalled==1)); n2 = float(np.sum(evs.recalled==0))      
        binsize=int(self.samplerate/5.0)
        
        if type(self.conn_type)==str:
            total_wins = cons_rec.shape[2]/binsize
            diff = cons_rec-cons_nrec
            if avg_epochs:
                diff_avg = np.reshape(diff[:, :, :binsize*total_wins],
                       (cons_rec.shape[0], cons_rec.shape[1], total_wins, binsize))
                diff_avg = np.mean(diff_avg, axis=3)
                fstats.append(diff_avg)
            else:
                fstats.append(diff)
        else:
            for i in range(len(cons_rec)):
                fstats.append([])            
                total_wins = cons_rec[i].shape[2]/binsize
                diff = cons_rec[i]-cons_nrec[i]
                if avg_epochs:
                    diff_avg = np.reshape(diff[:, :, :binsize*total_wins],
                       (diff.shape[0], diff.shape[1], total_wins, binsize))
                    diff_avg = np.mean(diff_avg, axis=3)
                    fstats[i].append(diff_avg)
                else:
                    fstats[i].append(diff)
        
        #Load preshuffled vectors
        shuffled_vecs = np.load('/scratch/esolo/mne_analysis/'+sub+'/shuffled_vecs.npy')
        
        for k in range(nRandomizations):
            
            if match_trial_n:
                idxs = np.arange(len(rec_idxs)+len(nrec_idxs))
                np.random.shuffle(idxs)
                n1 = len(rec_idxs)
            else:
                idxs = shuffled_vecs[k, :]
                
            null_rec, _, _, _, _ = mne.connectivity.spectral_connectivity(arr[idxs[:int(n1)]],
            method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax, indices=conn_inds,
            faverage=False, n_jobs=1, verbose=False, cwt_frequencies=cwtfreq, cwt_n_cycles=6.0)

            null_nrec, _, _, _, _ = mne.connectivity.spectral_connectivity(arr[idxs[int(n1):]],
            method=self.conn_type, mode=self.mode, sfreq=sfreq, fmin=fmin, fmax=fmax, indices=conn_inds,
            faverage=False, n_jobs=1, verbose=False, cwt_frequencies=cwtfreq, cwt_n_cycles=6.0)
            
            #Get average of the difference in 200ms windows
            if type(self.conn_type)==str:
                null_diff = null_rec-null_nrec
                if avg_epochs:
                    null_diff_avg = np.reshape(null_diff[:, :, :binsize*total_wins],
                                              (null_diff.shape[0], null_diff.shape[1], total_wins, binsize))

                    null_diff_avg = np.mean(null_diff_avg, axis=3)
                    fstats.append(null_diff_avg)
                else:
                    fstats.append(null_diff)
            else:
                for i in range(len(null_rec)):

                    null_diff = null_rec[i]-null_nrec[i]
                    if avg_epochs:
                        null_diff_avg = np.reshape(null_diff[:, :, :binsize*total_wins],
                              (null_diff.shape[0], null_diff.shape[1], total_wins, binsize))
                        null_diff_avg = np.mean(null_diff_avg, axis=3)
                        fstats[i].append(null_diff_avg)
                    else:
                        fstats[i].append(null_diff)
            print k
        
        fstats = np.array(fstats)    
        self.fstats = fstats

        import os
        fn = self.root+''+sub+'/'+sub+'_mne_MTL_AvgRef_FullSample_'+str(self.conn_type)+'_'+self.mode+'_dstat_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        self.stats_file = fn

        try:
            np.save(fn, fstats)
        except:
            try:
                os.mkdir(self.root+'/'+sub+'/')
                np.save(fn, fstats)
            except:
                np.save(fn, fstats)
                
    def pool_elecs_MTL(self, shuffle_func, nrand=500, pyFRsubs=pyFRsubs):
        import itertools
        #Just pool electrodes from MTL regions
        
        #Define MTL matrix labels
        Hipp_locTag = ['Left CA1','Left CA2','Left CA3','Left DG','Left Sub',
        'Right CA1','Right CA2','Right CA3','Right DG','Right Sub']
        MTL_locTag = ['Left PRC','Right PRC','Right EC','Right PHC','Left EC','Left PHC']

        MTL_labels = ['Left CA1',
         'Left CA2',
         'Left CA3',
         'Left DG',
         'Left Sub',
         'Left PRC',
         'Left EC',
         'Left PHC',
         'Right CA1',
         'Right CA2',
         'Right CA3',
         'Right DG',
         'Right Sub',
         'Right PRC',
         'Right EC',
         'Right PHC']
        
        #Skip pyFR (for now)
        if self.s in pyFRsubs:
            return 

        try:
            #Load the data
            M = np.load(self.root+'/'+self.s+'/'+self.s+'_mne_'+self.conn_type+'_'+self.mode+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy')
        except:
            return 
        
        rem = M[0]; nrem = M[1]

        #Get electrode data
        tal_path = str('/data/eeg/'+self.s+'/tal/'+self.s+'_talLocs_database_bipol.mat')
        tal_reader = TalReader(filename=tal_path)
        monopolar_channels = tal_reader.get_monopolar_channels()
        bipolar_pairs = tal_reader.get_bipolar_pairs()
        sub_elecs = tal_reader.tal_struct_array      

        if nrand==0:
            adj = np.zeros([2, len(MTL_labels), len(MTL_labels), rem.shape[2], rem.shape[3]])
            adj_sub = np.zeros([2, len(sub_elecs), len(sub_elecs), rem.shape[2], rem.shape[3]])
        else:
            adj = np.zeros([nrand+1, len(MTL_labels), len(MTL_labels)])
            adj_sub = np.zeros([nrand+1, len(sub_elecs), len(sub_elecs)])

        adjC = np.zeros([len(MTL_labels), len(MTL_labels)])
        combs = list(itertools.combinations(range(0, len(sub_elecs)), 2))            

        #Figure out which electrodes to exclude due to shared monopolar contacts
        pairs_to_skip = []
        for c in combs:
            t = list(sub_elecs[c[0]]['channel'])
            t.extend(sub_elecs[c[1]]['channel'])
            if len(np.unique(t))<4:
                pairs_to_skip.append(c)
        
        Md = shuffle_func(rem, nrem, nrand)

        for c in combs:

            if c in pairs_to_skip:
                continue

            elec1 = c[0]
            elec2 = c[1]

            #Ignore comparison if doesn't show up in usable list
            try:
                region1_idx = MTL_labels.index(sub_elecs[elec1]['locTag'])
                region2_idx = MTL_labels.index(sub_elecs[elec2]['locTag'])
            except ValueError:
                continue

            #Need to sort region indices to use only upper-triangle of matrix
            r_list = [region1_idx, region2_idx]
            r_list.sort()
            
            #Sum across electrodes
            adj[:, r_list[0], r_list[1], :, :] += Md[:, elec2, elec1, :, :]
            adj_sub[:, elec1, elec2, :, :] += Md[:, elec2, elec1, :, :]
            adjC[r_list[0], r_list[1]] += 1
            
            print c

        self.stats_file = self.root+'/'+self.s+'/'+self.s+'_MTL_mne_'+self.conn_type+'_'+self.mode+'_'+shuffle_func.__name__+'_'+str(self.win_st)+'-'+str(self.win_fin)+'_'+self.band+'.npy'
        np.save(self.stats_file, np.array([adj, adjC]))
        return
                
    def get_average_network(self):
        #For if you don't want to do an SEM comparison, and just extract average networks over all trials
        
        if self.mode=='cwt_morlet':
            if self.band=='theta':
                freq=[3., 8.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 1)
            elif self.band=='beta':
                freq=[15., 25.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='lowgamma':
                freq=[30., 40.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            elif self.band=='highgamma':
                freq=[90., 105.]
                cwtfreq = np.arange(freq[0], freq[1]+1, 2)
            else:
                raise ValueError
        else:
            if self.band=='theta':
                freq=[3., 8.]
            elif self.band=='beta':
                freq=[15., 25.]
            elif self.band=='lowgamma':
                freq=[30., 40.]
            elif self.band=='highgamma':
                freq=[90., 105.]
            else:
                raise ValueError

        sub = self.s
        
        
